
--------------------------------------------------------------------------------------------
-- Practica de Enginyeria del Software II / Curs 2003 - 2004                              --
-- Simulador i Controlador de Transit en Temps Real                                       --
--                                                                                        --
-- Ciutats.adb: En aquest modul es crea un tipus de dades Ciutat on s'agrupa tota la      --
--              informacio relativa a la ciutat del simulador: carrers, posicions de      --
--              vehicles, semafors, cruilles, dades de la ciutat, etc. Es disposa de      --
--              tots els metodes necessaris per a que les tasques interactuin de forma    --
--              abstracta amb la ciutat. Conte a mes els metodes necessaris per cridar    --
--              als procediments d'actualitzacio de la interficie del modul corresponent. --
-- Realitzat per: Francisco Jose Aguilar Celdran                                          --
--                Sergio Blanco Cuaresma                                                  --
--------------------------------------------------------------------------------------------


with Ada.Exceptions; use Ada.Exceptions;
with interficievisual; use interficievisual;
with Glib;              use Glib;
with Ada.Text_IO; use Ada.Text_IO;
with Ada.Integer_Text_IO; use Ada.Integer_Text_IO;

package body Ciutats is

   --------------------------------
   -- Establir_Parametres_Ciutat --
   --------------------------------

   procedure Establir_Parametres_Ciutat (C: in out Ciutat; Separacio_Cruilles, Uts,
                                         Temps_Contenidor: in Positive; Periode_Semafors,
                                         Periode_Escombraries: in Positive; Pos_Garatge,
                                         Pos_Taller: Coordenada; Carrer_Taller, Carrer_Garatge:
                                         Positive; Direccio_Garatge, Direccio_Taller: Direccio;
                                         Ample_Carrer: Positive;Controlar_Trafic: Boolean) is

   begin
      C.Separacio_Cruilles := Separacio_Cruilles;
      C.Periode_Semafors := Periode_Semafors;
      C.Uts := Uts;
      C.Periode_Escombraries := Periode_Escombraries;
      C.Minuts_Per_Contenidor := Temps_Contenidor;
      C.Amplada_Carrers := Ample_Carrer;
      C.Longitud_H_Ciutat := (C.Separacio_Cruilles * C.Num_Carrers_V) + (C.Separacio_Cruilles * C.Num_Carrers_V);
      C.Longitud_V_Ciutat := (C.Separacio_Cruilles * C.Num_Carrers_H) + (C.Separacio_Cruilles * C.Num_Carrers_H);
      C.Coord_Garatge := Pos_Garatge;
      C.Coord_Taller := Pos_Taller;
      C.Carrer_Garatge := Carrer_Garatge;
      C.Carrer_Taller := Carrer_Taller;
      C.Direccio_Garatge := Direccio_Garatge;
      C.Direccio_Taller := Direccio_Taller;
      C.TempsInici := Clock;
      C.Control_Trafic_Activat := Controlar_Trafic;
      C.Accions := new Interficie_Protegida;
      C.Accions.Inicialitzacio;
   end Establir_Parametres_Ciutat;

   ---------------------------------
   -- Establir_Parametres_Vehicle --
   ---------------------------------

   procedure Establir_Parametres_Vehicle (C: in out Ciutat; Vehicle: in Tipus_Vehicle;
                                          Velocitat, Distancia, Desviacio: in Natural;
                                          Probabilitat: in Float; Num_Inicial: in Natural) is
   begin
      C.Velocitat(Vehicle) := Velocitat;
      C.Distancia_Vehicle(Vehicle) := Distancia;
      C.Desviacio(Vehicle) := Desviacio;
      C.Probabilitat(Vehicle) := Probabilitat;
      C.Nombre_Inicial(Vehicle) := Num_Inicial;
   end Establir_Parametres_Vehicle;

   ----------------------------------
   -- Establir_Coordenades_Cruilla --
   ----------------------------------

   procedure Establir_Coordenades_Cruilla (C: in out Ciutat; Fila, Columna: in Positive; 
                                           Coord: in Coordenades_Cruilla) is
   begin
      C.Pos_Cruilles(Fila,Columna) := Coord;
   end Establir_Coordenades_Cruilla;

   -----------------------------------
   -- Establir_Temps_Control_Trafic --
   -----------------------------------
   
   procedure Establir_Temps_Control_Trafic(C: in out Ciutat; Divisor_Temps_Controlador: in Positive) is
   begin
      C.Divisor_Minuts_Controlador := Divisor_Temps_Controlador;
   end Establir_Temps_Control_trafic;


   ------------------------------
   -- Establir_Sentits_Carrers --
   ------------------------------

   procedure Establir_Sentits_Carrers (C: in out Ciutat; Sentit_H_Ini: Orientacio_Horitzontal;
                                       Sentit_V_Ini: Orientacio_Vertical) is
      Sentit_H_Actual: Orientacio_Horitzontal;
      Sentit_V_Actual: Orientacio_Vertical;
   begin
      Sentit_H_Actual := Sentit_H_Ini;
      Sentit_V_Actual := Sentit_V_Ini;
      for I in 1..C.Num_Carrers_H loop
         C.Sentits_Carrers_H(I) := Sentit_H_Actual;
         case Sentit_H_Actual is
            when Esquerra =>
               Sentit_H_Actual := Dreta;
            when Dreta =>
               Sentit_H_Actual := Esquerra;
         end case;
      end loop;

      for I in 1..C.Num_Carrers_V loop
         C.Sentits_Carrers_V(I) := Sentit_V_Actual;
         case Sentit_V_Actual is
            when Amunt =>
               Sentit_V_Actual := Abaix;
            when Abaix =>
               Sentit_V_Actual := Amunt;
         end case;
      end loop;
   end Establir_Sentits_Carrers;


   -----------------------------
   -- Establir_Tamany_Vehicle --
   -----------------------------

   procedure Establir_Tamany_Vehicle (C: in out Ciutat; Tamany: in Positive; Tipus: in Tipus_Vehicle) is
   begin
      case Tipus is
         when Cotxe => C.Tamany_Cotxe := Tamany;
         when Camio => C.Tamany_Camio := Tamany;
         when Bicicleta => C.Tamany_Bicicleta := Tamany;
         when Ciclomotor => C.Tamany_Ciclomotor := Tamany;

      end case;
   end Establir_Tamany_Vehicle;


   ---------------------------
   -- Meitat_Tamany_Vehicle --
   ---------------------------

   function Meitat_Tamany_Vehicle (C: in Ciutat; Tipus: in Tipus_Vehicle) return Positive is
   begin
      case Tipus is
         when Cotxe => return C.Tamany_Cotxe;
         when Camio => return C.Tamany_Camio;
         when Bicicleta => return C.Tamany_Bicicleta;
         when Ciclomotor => return C.Tamany_Ciclomotor;
      end case;
   end Meitat_Tamany_Vehicle;

     
   ---------------------
   -- Nombre_Vehicles --
   ---------------------

   function Nombre_Vehicles (C: in Ciutat; Tipus: Tipus_Vehicle) return Natural is
   begin
      return C.Nombre_Inicial(Tipus);
   end Nombre_Vehicles;

   ------------------------------
   -- Separacio_Entre_Cruilles --
   ------------------------------

   function Separacio_Entre_Cruilles (C: in Ciutat) return Positive is
   begin
      return C.Separacio_Cruilles;
   end Separacio_Entre_Cruilles;


   ----------------------------
   -- Unitat_Temps_Simulacio --
   ----------------------------

   function Unitat_Temps_Simulacio (C: in Ciutat) return Positive is
   begin
      return C.Uts;
   end Unitat_Temps_Simulacio;

   --------------------------------
   -- Periode_Camio_Escombraries --
   --------------------------------

   function Periode_Camio_Escombraries (C: in Ciutat) return Positive is
   begin
      return C.Periode_Escombraries;
   end Periode_Camio_Escombraries;

   -------------------------
   -- Temps_En_Contenidor --
   -------------------------

   function Temps_En_Contenidor (C: in Ciutat) return Natural is
   begin
      return C.Minuts_Per_Contenidor;
   end Temps_En_Contenidor;

   -------------------------
   -- Coordenades_Garatge --
   -------------------------

   function Coordenades_Garatge (C: in Ciutat) return Coordenada is
   begin
      return C.Coord_Garatge;
   end Coordenades_Garatge;


   ----------------------------
   -- Obtenir_Carrer_Garatge --
   ----------------------------

   procedure Obtenir_Carrer_Garatge (C: in Ciutat; Carrer: out Positive; Dir: out Direccio) is
   begin
      Carrer := C.Carrer_Taller;
      Dir := C.Direccio_Taller;
   end Obtenir_Carrer_Garatge;

   ------------------------
   -- Coordenades_Taller --
   ------------------------

   function Coordenades_Taller (C: in Ciutat) return Coordenada is
   begin
      return C.Coord_Taller;
   end Coordenades_Taller;

   ---------------------------
   -- Obtenir_Carrer_Taller --
   ---------------------------

   procedure Obtenir_Carrer_Taller (C: in Ciutat; Carrer: out Positive; Dir: out Direccio) is
   begin
      Carrer := C.Carrer_Taller;
      Dir := C.Direccio_Taller;
   end Obtenir_Carrer_Taller;

   -----------------
   -- Num_Carrers --
   -----------------

   function Num_Carrers (C: in Ciutat; D: in Direccio) return Positive is
   begin
      if D = Horitzontal then
         return C.Num_Carrers_H;
      else
         return C.Num_Carrers_V;
      end if;
   end Num_Carrers;

   ------------------
   -- Ample_Carrer --
   ------------------

   function Ample_Carrer (C: in Ciutat) return Positive is
   begin
      return C.Amplada_Carrers;
   end Ample_Carrer;

   -------------------------
   -- Num_Cruilles_Ciutat --
   -------------------------

   function Num_Cruilles_Ciutat (C: in Ciutat) return Positive is
   begin
      return C.Num_Cruilles;
   end Num_Cruilles_Ciutat;


   ----------------------
   -- Velocitat_Maxima --
   ----------------------

   function Velocitat_Maxima (C: in Ciutat; Tipus: Tipus_Vehicle) return Positive is
   begin
      return C.Velocitat(Tipus);
   end Velocitat_Maxima;

   -------------------------------
   -- Probabilitat_Seguir_Recte --
   -------------------------------

   function Probabilitat_Seguir_Recte (C: in Ciutat; Tipus: Tipus_Vehicle) return Float is
   begin
      return C.Probabilitat(Tipus);
   end Probabilitat_Seguir_Recte;

   -----------------------------
   -- Distancia_Mitja_Vehicle --
   -----------------------------

   function Distancia_Mitja_Vehicle (C: in Ciutat; Tipus: Tipus_Vehicle) return Positive is
   begin
      return C.Distancia_Vehicle(Tipus);
   end Distancia_Mitja_Vehicle;

   ---------------------------------
   -- Desviacio_Distancia_Vehicle --
   ---------------------------------

   function Desviacio_Distancia_Vehicle (C: in Ciutat; Tipus: Tipus_Vehicle) return Integer is
   begin
      return C.Desviacio(Tipus);
   end Desviacio_Distancia_Vehicle;

   ----------------------------------
   -- Obtenir_Temps_Control_Trafic --
   ----------------------------------
   
   function Obtenir_Temps_Control_Trafic(C: in Ciutat) return Positive is
   begin
      return C.Divisor_Minuts_Controlador;
   end Obtenir_Temps_Control_Trafic;
   

   ---------------------------------
   -- Obtenir_Coordenades_Cruilla --
   ---------------------------------

   procedure Obtenir_Coordenades_Cruilla (C: in Ciutat; Id: in Positive; X, Y: out Positive) is
      Columnes : Positive;
   begin
      Columnes := C.Num_Carrers_V;
      X := ((Id - 1) / Columnes) + 1;
      Y := ((Id - 1) mod Columnes) + 1;
   end Obtenir_Coordenades_Cruilla;

   --------------------------
   -- Inicialitzar_Cruilla --
   --------------------------

   procedure Inicialitzar_Cruilla (C: in out Ciutat; Fila, Columna: in Positive;
                                   Color_H, Color_V: in Estat_Semafor) is
   begin
      Establir_Colors_Inicials(C.Taula_Cruilles(Fila,Columna),Color_H,Color_V);
   end Inicialitzar_Cruilla;


   -----------------------
   -- Alternar_Semafors --
   -----------------------

   procedure Alternar_Semafors (C: in out Ciutat; Fila, Columna: in Positive) is
   begin
      Alternar_Semafors_Cruilla(C.Taula_Cruilles(Fila,Columna));
   end Alternar_Semafors;

   -----------------------
   -- Travessar_Semafor --
   -----------------------

   procedure Travessar_Semafor (C: in out Ciutat; Fila, Columna: in Positive; D: in Direccio) is
   begin
      if D = Horitzontal then
         Travessar_Semafor_Horitzontal(C.Taula_Cruilles(Fila,Columna));
      else
         Travessar_Semafor_Vertical(C.Taula_Cruilles(Fila,Columna));  
      end if;
   end Travessar_Semafor;


   --------------------------
   -- Estat_Actual_Semafor --
   --------------------------

   function Estat_Actual_Semafor (C: in Ciutat; Fila, Columna: in Positive; D: in Direccio) 
                                  return Estat_Semafor is
   begin
      if D = Horitzontal then
         return Color_Semafor_Horitzontal(C.Taula_Cruilles(Fila,Columna));
      else
         return Color_Semafor_Vertical(C.Taula_Cruilles(Fila,Columna));
      end if;
   end Estat_Actual_Semafor;


   ------------------------------
   -- Obtenir_Periode_Semafors --
   ------------------------------

   function Obtenir_Periode_Semafors (C: in Ciutat) return Positive is
   begin
      return C.Periode_Semafors;
   end Obtenir_Periode_Semafors;


   -------------------------------
   -- Establir_Periode_Semafors --
   -------------------------------

   procedure Establir_Periode_Semafors (C: in out Ciutat; Periode: Positive) is
   begin
      C.Periode_Semafors := Periode;
   end Establir_Periode_Semafors;


   -----------------------------   
   -- Modificar_Estat_Semafor -- 
   -----------------------------

   procedure Modificar_Estat_Semafor (C: in out Ciutat; Fila, Columna: in Positive;
                                      D: in Direccio; Color: in Estat_Semafor) is
   begin
      if D = Horitzontal then
         Modificar_Semafor_Horitzontal(C.Taula_Cruilles(Fila,Columna),Color);
      else
         Modificar_Semafor_Vertical(C.Taula_Cruilles(Fila,Columna),Color);
      end if;
   end Modificar_Estat_Semafor;

   ------------------------------
   -- Vehicles_Aturats_Semafor --
   ------------------------------

   function Vehicles_Aturats_Semafor (C: in Ciutat; Fila, Columna: in Positive;
                                      D: in Direccio) return Natural is
   begin
      if D = Horitzontal then
         return Nombre_Vehicles_Aturats_Horitzontal(C.Taula_Cruilles(Fila,Columna));
      else
         return Nombre_Vehicles_Aturats_Vertical(C.Taula_Cruilles(Fila,Columna));
      end if;
   end Vehicles_Aturats_Semafor;


   ---------------------------
   -- Control_Trafic_Ciutat --   
   ---------------------------

   function Control_Trafic_Ciutat (C: in Ciutat) return Boolean is
   begin
      return C.Control_Trafic_Activat;
   end Control_Trafic_Ciutat;

   ------------------------
   -- Vehicle_En_Cruilla --
   ------------------------
   procedure Vehicle_En_Cruilla (C: in Ciutat; Pos_X, Pos_Y : in Natural; Pos_Cruilla:
                                 out Coordenada; Dir: in Direccio; Num_Carrer: Positive;
                                 En_Cruilla: out Boolean; Metres_A_Semafor: out Natural;
                                 Meitat_Tam_Vehicle: in Positive) is
      Trobat : Boolean := False;
      Fila : Positive := 1;
      Columna : Positive := 1;
      Sentit_H : Orientacio_Horitzontal;
      Sentit_V : Orientacio_Vertical;
      Carrer_V, Carrer_H : Natural;
   begin
      En_Cruilla := False;
      Metres_A_Semafor := 500;
      if (Dir = Horitzontal) then
         Sentit_H := Orientacio_Carrer_H(C,Num_Carrer);
         if Sentit_H = Dreta then
            Carrer_V := 1;
            while Carrer_V <= C.Num_Carrers_V and (not Trobat) loop
               if C.Pos_Cruilles(Num_Carrer,Carrer_V).Parada_H.X = (Pos_X + Meitat_Tam_Vehicle) then
                  En_Cruilla := True;
                  Pos_Cruilla.X := Num_Carrer;
                  Pos_Cruilla.Y := Carrer_V;
                  Metres_A_Semafor := 0;
                  Trobat := True;
               elsif C.Pos_Cruilles(Num_Carrer,Carrer_V).Parada_H.X > Pos_X + Meitat_Tam_Vehicle then
                  Metres_A_Semafor := C.Pos_Cruilles(Num_Carrer,Carrer_V).Parada_H.X - (Pos_X + Meitat_Tam_Vehicle);
                  Trobat := True;
               end if;
               Carrer_V := Carrer_V + 1;
            end loop;
         else
            Carrer_V := C.Num_Carrers_V;
            while Carrer_V > 0 and (not Trobat) loop
               if C.Pos_Cruilles(Num_Carrer,Carrer_V).Parada_H.X = (Pos_X - Meitat_Tam_Vehicle) then
                  En_Cruilla := True;
                  Pos_Cruilla.X := Num_Carrer;
                  Pos_Cruilla.Y := Carrer_V;
                  Metres_A_Semafor := 0;
                  Trobat := True;
               elsif C.Pos_Cruilles(Num_Carrer,Carrer_V).Parada_H.X < (Pos_X - Meitat_Tam_Vehicle) then
                  Metres_A_Semafor := (Pos_X - Meitat_Tam_Vehicle) - C.Pos_Cruilles(Num_Carrer,Carrer_V).Parada_H.X;
                  Trobat := True;
               end if;
               Carrer_V := Carrer_V - 1;
            end loop;
         end if;
      else
         Sentit_V := Orientacio_Carrer_V(C,Num_Carrer);
         if Sentit_V = Abaix then
            Carrer_H := 1;
            while Carrer_H <= C.Num_Carrers_H and (not Trobat) loop
               if C.Pos_Cruilles(Carrer_H,Num_Carrer).Parada_V.Y = (Pos_Y + Meitat_Tam_Vehicle) then
                  En_Cruilla := True;
                  Pos_Cruilla.X := Carrer_H;
                  Pos_Cruilla.Y := Num_Carrer;
                  Metres_A_Semafor := 0;
                  Trobat := True;
               elsif C.Pos_Cruilles(Carrer_H,Num_Carrer).Parada_V.Y > (Pos_Y + Meitat_Tam_Vehicle) then
                  Metres_A_Semafor := C.Pos_Cruilles(Carrer_H,Num_Carrer).Parada_V.Y - (Pos_Y + Meitat_Tam_Vehicle);
                  Trobat := True;
               end if;
               Carrer_H := Carrer_H + 1;
            end loop;
         else
            Carrer_H := C.Num_Carrers_H;
            while Carrer_H > 0 and (not Trobat) loop               
               if C.Pos_Cruilles(Carrer_H,Num_Carrer).Parada_V.Y = (Pos_Y - Meitat_Tam_Vehicle) then
                  En_Cruilla := True;
                  Pos_Cruilla.X := Carrer_H;
                  Pos_Cruilla.Y := Num_Carrer;
                  Metres_A_Semafor := 0;
                  Trobat := True;
               elsif C.Pos_Cruilles(Carrer_H,Num_Carrer).Parada_V.Y < (Pos_Y - Meitat_Tam_Vehicle) then
                  Metres_A_Semafor := (Pos_Y - Meitat_Tam_Vehicle) - C.Pos_Cruilles(Carrer_H,Num_Carrer).Parada_V.Y;
                  Trobat := True;
               end if;               
               Carrer_H := Carrer_H - 1;
            end loop;
         end if;         
      end if;

   exception
      when E: others => Put_Line("Excepcio en Vehicle_En_Cruilla " & Exception_Name(E) & ", causada per: " & Exception_Message(E));
   end Vehicle_En_Cruilla;


   ---------------------------------   
   -- Coordenades_Primera_Cruilla --
   ---------------------------------
   function Coordenades_Primera_Cruilla (C: in Ciutat; Dir: in Direccio) return Coordenada is
   begin
      if Dir = Horitzontal then
         return C.Pos_Cruilles(1,1).Parada_H;
      else
         return C.Pos_Cruilles(1,1).Parada_V;
      end if;
   end Coordenades_Primera_Cruilla;


   -------------------------
   -- Orientacio_Carrer_H --
   -------------------------   

   function Orientacio_Carrer_H (C: in Ciutat; Carrer: Positive) return Orientacio_Horitzontal is
   begin
      return C.Sentits_Carrers_H(Carrer);
   end Orientacio_Carrer_H;   


   -------------------------
   -- Orientacio_Carrer_V --
   -------------------------

   function Orientacio_Carrer_V (C: in Ciutat; Carrer: Positive) return Orientacio_Vertical is
   begin
      return C.Sentits_Carrers_V(Carrer);
   end Orientacio_Carrer_V;

   ------------------------------
   -- Carrers_Entre_Coordenada --
   ------------------------------

   procedure Carrers_Entre_Coordenada (C: in Ciutat; Pos: in Coordenada; C1, C2: out Positive;
                                       D: in Direccio) is
      Cont : Positive := 1;
   begin
      if D = Horitzontal then
         while C.Pos_Cruilles(1,Cont).Parada_V.Y < Pos.Y loop
            Cont := Cont + 1;
         end loop;
         C2 := Cont;
         C1 := Cont - 1;
      else
         while C.Pos_Cruilles(Cont,1).Parada_H.X < Pos.X loop
            Cont := Cont + 1;
         end loop;
         C2 := Cont;
         C1 := Cont - 1;            
      end if;
   end Carrers_Entre_Coordenada;


   ------------------
   -- Tam_Ciutat_H --
   ------------------

   function Tam_Ciutat_H (C: in Ciutat) return Positive is
   begin
      return C.Longitud_H_Ciutat;
   end Tam_Ciutat_H;


   ------------------
   -- Tam_Ciutat_V --
   ------------------

   function Tam_Ciutat_V (C: in Ciutat) return Positive is
   begin
      return C.Longitud_V_Ciutat;
   end Tam_Ciutat_V;

   --------------------------
   -- Interficie_Protegida --
   --------------------------

   protected body Interficie_Protegida is
      procedure Inicialitzacio is
         OK : Boolean;
      begin
         Inicialitzar_Accions (Accions_Interficie,OK);
      end Inicialitzacio;

      procedure Nova_Accio (Dada: in Info_Accio) is
         OK : Boolean;
      begin
         Afegir_Accio(Accions_Interficie,Dada,Ok);
         Num_Accions := Num_Accions + 1;
      end Nova_Accio;

      entry Extreure_Accio (Dada: out Info_Accio) when Num_Accions > 0 is
         OK : Boolean;
      begin
         Obtenir_Accio(Accions_Interficie,Dada,Ok);
         Num_Accions := Num_Accions - 1;
      end Extreure_Accio;

      function Hi_Ha_Accions return boolean is
      begin
         return Queden_Accions(Accions_Interficie);
      end Hi_Ha_Accions; 
   end Interficie_Protegida;


   ----------------------------- 
   -- Inicialitzar_Info_Accio --
   -----------------------------

   function Inicialitzar_Info_Accio return Info_Accio is
      Info : Info_Accio;
      Coord : Coordenada;
   begin
      Info.Tipus := Moure_Vehicle;
      Info.Id := 1;
      Info.Vehicle := Cotxe;
      Info.Grua := False;
      Info.Escombraries := False;
      Coord.X := 1;
      Coord.Y := 1;
      Info.Posicio_Vehicle := Coord;
      Info.Num_Cruilla := Coord;
      Info.Semafor_Verd := Horitzontal;
      return Info;
   end Inicialitzar_Info_Accio;

   -------------------
   -- Moure_Vehicle --
   -------------------

   procedure Moure_Vehicle (C: in out Ciutat; Pos_Actual: in Coordenada; Id: in Positive;
                            Tipus: in Tipus_Vehicle; Grua, Escombraries: in Boolean) is
      Dades : Info_Accio;
   begin
      Dades := Inicialitzar_Info_Accio;
      Dades.Tipus := Moure_Vehicle;
      Dades.Id := Id;
      Dades.Vehicle := Tipus;
      Dades.Posicio_Vehicle := Pos_Actual;
      if Grua then
         Dades.Grua := True;
      end if;
      if Escombraries then
         Dades.Escombraries := True;
      end if;
      C.Accions.Nova_Accio(Dades);
   end Moure_Vehicle;

   

   -------------------
   -- Vehicle_Avariat --
   -------------------

   procedure Vehicle_Avariat (C: in out Ciutat; Pos_Actual: in Coordenada; Id: in Positive;
                            Tipus: in Tipus_Vehicle) is
      Dades : Info_Accio;
   begin
      Dades := Inicialitzar_Info_Accio;
      Dades.Tipus := Vehicle_Avariat;
      Dades.Id := Id;
      Dades.Vehicle := Tipus;
      Dades.Posicio_Vehicle := Pos_Actual;
      C.Accions.Nova_Accio(Dades);
   end Vehicle_Avariat;


   ---------------------------------
   -- Canviar_Semafors_Interficie --
   ---------------------------------

   procedure Canviar_Semafors_Interficie (C: in out Ciutat; Fila, Columna: in Positive;
                                          D: in Direccio; Color: in Estat_Semafor) is
      Dades : Info_Accio;
   begin
      Dades := Inicialitzar_Info_Accio;
      Dades.Tipus := Canviar_Semafors;
      Dades.Num_Cruilla.X := Fila;
      Dades.Num_Cruilla.Y := Columna;
      if Color = Verd then
         if D = Horitzontal then
            Dades.Semafor_Verd := Horitzontal;
         else
            Dades.Semafor_Verd:= Vertical;
         end if;
      else
         if D = Horitzontal then
            Dades.Semafor_Verd := Vertical;
         else
            Dades.Semafor_Verd := Horitzontal;
         end if;
      end if;
      C.Accions.Nova_Accio(Dades);
   end Canviar_Semafors_Interficie;


   ----------------------------
   -- Treure_Vehicle_Garatge --
   ----------------------------

   procedure Treure_Vehicle_Garatge (C: in out Ciutat; Grua, Escombraries: Boolean) is
      Dades : Info_Accio;
   begin
      Dades := Inicialitzar_Info_Accio;
      Dades.Tipus := Treure_Del_Garatge;
      if Grua then
         Dades.Grua := True;
      elsif Escombraries then
         Dades.Escombraries := True;
      end if;
      C.Accions.Nova_Accio(Dades);
   end Treure_Vehicle_Garatge;

   -----------------------------
   -- Guardar_Vehicle_Garatge --
   -----------------------------

   procedure Guardar_Vehicle_Garatge (C: in out Ciutat; Grua, Escombraries: Boolean) is
      Dades : Info_Accio;
   begin
      Dades := Inicialitzar_Info_Accio;
      Dades.Tipus := Guardar_En_Garatge;
      if Grua then
         Dades.Grua := True;
      elsif Escombraries then
         Dades.Escombraries := True;
      end if;
      C.Accions.Nova_Accio(Dades);      
   end Guardar_Vehicle_Garatge;


   ------------------------------
   -- Recollir_Vehicle_Avariat --
   ------------------------------

   procedure Recollir_Vehicle_Avariat (C: in out Ciutat; Pos_Actual: in Coordenada;
                                       Id: Positive; Tipus: Tipus_Vehicle) is
      Dades : Info_Accio;
   begin
      Dades := Inicialitzar_Info_Accio;
      Dades.Tipus := Recollir_Avariat;
      Dades.Posicio_Vehicle := Pos_Actual;
      Dades.Id := Id;
      Dades.Vehicle := Tipus;
      C.Accions.Nova_Accio(Dades);
   end Recollir_Vehicle_Avariat;   


   ------------------------------
   -- Deixar_Avariat_En_Taller --
   ------------------------------

   procedure Deixar_Avariat_En_Taller (C: in out Ciutat; Id: Positive; Tipus: Tipus_Vehicle) is
      Dades : Info_Accio;
   begin
      Dades := Inicialitzar_Info_Accio;
      Dades.Tipus := Avariat_A_Taller;
      Dades.Id := Id;
      Dades.Vehicle := Tipus;
      C.Accions.Nova_Accio(Dades);
   end Deixar_Avariat_En_Taller;
   
   
   -- Estadstiques de fluidesa
   procedure Actualitzar_Flow (C: in out Ciutat; Flow: Duration) is
      Dades : Info_Accio;
      Ara: Time;
   begin
      C.FlowAverage := (C.FlowAverage + Flow) / 2;
      Dades := Inicialitzar_Info_Accio;
      Dades.Tipus := Actualitzar_Flow;
      
      Ara := Clock;
      Dades.Valor := ( ((To_Duration(Ara - C.TempsInici) - C.FlowAverage) / To_Duration((Ara - C.TempsInici)))) * 100;
      
      C.Accions.Nova_Accio(Dades);
   end;


   -------------------------
   -- Accions_Per_Tractar --
   -------------------------

   function Accions_Per_Tractar(C: in Ciutat) return boolean is
   begin
      return C.Accions.Hi_Ha_Accions;
   end Accions_Per_Tractar;


   --------------------------------
   -- Esperar_Accions_Interficie --
   --------------------------------

   procedure Esperar_Accions_Interficie (C: in out Ciutat; Dades: out Info_Accio) is
   begin
      C.Accions.Extreure_Accio(Dades);
   end Esperar_Accions_Interficie;


   ----------------------------
   -- Actualitzar_Interficie --
   ----------------------------

   procedure Actualitzar_Interficie (Dades: in Info_Accio) is
      Vehicle: TVehicle;
   begin
      
      if (Dades.Tipus = Moure_Vehicle) then
         Vehicle.Id := Dades.Id;
         Vehicle.X := Gint(Dades.Posicio_Vehicle.X);
         Vehicle.Y := Gint(Dades.Posicio_Vehicle.Y);
         Vehicle.Avariat := false;
         
         if (Dades.Grua) then
            MoveCamioGrua(Vehicle, Dades.Vehicle);
         elsif (Dades.Escombraries) then
            MoveCamioBrossa(Vehicle, Dades.Vehicle);
         else
            MoveVehicle(Vehicle, Dades.Vehicle);
         end if;
      
      elsif (Dades.Tipus = Vehicle_Avariat) then
         Vehicle.Id := Dades.Id;
         Vehicle.X := Gint(Dades.Posicio_Vehicle.X);
         Vehicle.Y := Gint(Dades.Posicio_Vehicle.Y);
         Vehicle.Avariat := true;
         MoveVehicle(Vehicle, Dades.Vehicle);

      elsif (Dades.Tipus = Canviar_Semafors) then
         if (Dades.Semafor_Verd = Horitzontal) then
            -- Per la interficie el primer semafor es el (0,0), per la logica
            -- es el (1,1) aixi que restem 1.
            ChangeSemaphore(Gint(Dades.Num_Cruilla.X - 1), Gint(Dades.Num_Cruilla.Y - 1), true);
         else
            -- Per la interficie el primer semafor es el (0,0), per la logica
            -- es el (1,1) aixi que restem 1.
            ChangeSemaphore(Gint(Dades.Num_Cruilla.X - 1), Gint(Dades.Num_Cruilla.Y - 1), false);
         end if;

      elsif (Dades.Tipus = Guardar_En_Garatge) then
         if (Dades.Grua) then
            CamioGruaToGarage;
         elsif (Dades.Escombraries) then
            CamioBrossaToGarage;
         end if;
         
      elsif (Dades.Tipus = Recollir_Avariat) then
         Vehicle.Id := Dades.Id;
         CamioGruaTakeCar(Vehicle, Dades.Vehicle);
      elsif (Dades.Tipus = Avariat_A_Taller) then
         CamioGruaLeaveCar;
      elsif (Dades.Tipus = Treure_Del_Garatge) then
         if (Dades.Grua) then
            CamioGruaToStreet;
         elsif (Dades.Escombraries) then
            CamioBrossaToStreet;
         end if;
      elsif (Dades.Tipus = Actualitzar_Flow) then
         Flow(duration'image(Dades.Valor));
      end if;

      -- Actualitzem buffer
      DrawCity;

      end Actualitzar_Interficie;

end Ciutats;



