
------------------------------------------------------------------------------------------
-- Practica de Enginyeria del Software II / Curs 2003 - 2004                            --
-- Simulador i Controlador de Transit en Temps Real                                     --
--                                                                                      --
-- Ciutats.ads: Fitxer descriptor del modul Ciutats, amb les especificacions dels       --
--              procediments, funcions, tipus de dades i objectes protegits que         --
--              s'empren en la implementacio d'aquest package.                          --
-- Realitzat per: Francisco Jose Aguilar Celdran                                        --
--                Sergio Blanco Cuaresma                                                --
------------------------------------------------------------------------------------------

with Ada.Real_Time; use Ada.Real_Time;
with Tipus_Comuns; use Tipus_Comuns;
with Cruilles; use Cruilles;
with Accions; use Accions;

package Ciutats is
   type Ciutat(Carrers_H, Carrers_V: Positive) is limited private;
   type Ciutat_ptr is access Ciutat;

   -- Procediments d'inicialitzacio de la la ciutat
   procedure Establir_Parametres_Vehicle (C: in out Ciutat; Vehicle: in Tipus_Vehicle;
                                          Velocitat, Distancia, Desviacio: in Natural;
                                          Probabilitat: in Float; Num_Inicial: in Natural);
   procedure Establir_Parametres_Ciutat (C: in out Ciutat; Separacio_Cruilles, Uts,
                                         Temps_Contenidor: in Positive; Periode_Semafors,
                                         Periode_Escombraries: in Positive; Pos_Garatge,
                                         Pos_Taller: Coordenada; Carrer_Taller, Carrer_Garatge:
                                         Positive; Direccio_Garatge, Direccio_Taller: Direccio;
                                         Ample_Carrer: Positive; Controlar_Trafic: Boolean);
   procedure Establir_Coordenades_Cruilla (C: in out Ciutat; Fila, Columna: in Positive; 
                                           Coord: in Coordenades_Cruilla);  
   procedure Establir_Sentits_Carrers (C: in out Ciutat; Sentit_H_Ini: Orientacio_Horitzontal;
                                       Sentit_V_Ini: Orientacio_Vertical);
   procedure Establir_Tamany_Vehicle (C: in out Ciutat; Tamany: in Positive; Tipus: in Tipus_Vehicle);
   procedure Establir_Temps_Control_Trafic(C: in out Ciutat; Divisor_Temps_Controlador: in Positive);

   -- Funcions consultores d'informacio de la ciutat
   function Meitat_Tamany_Vehicle (C: in Ciutat; Tipus: in Tipus_Vehicle) return Positive;
   function Nombre_Vehicles (C: in Ciutat; Tipus: in Tipus_Vehicle) return Natural;
   function Separacio_Entre_Cruilles (C: in Ciutat) return Positive;  
   function Unitat_Temps_Simulacio (C: in Ciutat) return Positive;
   function Periode_Camio_Escombraries (C: in Ciutat) return Positive;
   function Temps_En_Contenidor (C: in Ciutat) return Natural;
   function Coordenades_Garatge (C: in Ciutat) return Coordenada;
   procedure Obtenir_Carrer_Garatge (C: in Ciutat; Carrer: out Positive; Dir: out Direccio);
   function Coordenades_Taller (C: in Ciutat) return Coordenada;
   procedure Obtenir_Carrer_Taller (C: in Ciutat; Carrer: out Positive; Dir: out Direccio);
   function Num_Carrers (C: in Ciutat; D: in Direccio) return Positive;
   function Ample_Carrer (C: in Ciutat) return Positive;
   function Num_Cruilles_Ciutat (C: in Ciutat) return Positive;
   procedure Carrers_Entre_Coordenada (C: in Ciutat; Pos: in Coordenada; C1, C2: out Positive;
                                       D: in Direccio);
   procedure Obtenir_Coordenades_Cruilla (C: in Ciutat; Id: in Positive; X, Y: out Positive);
   function Velocitat_Maxima (C: in Ciutat; Tipus: Tipus_Vehicle) return Positive;
   function Probabilitat_Seguir_Recte (C: in Ciutat; Tipus: Tipus_Vehicle) return Float;
   function Distancia_Mitja_Vehicle (C: in Ciutat; Tipus: Tipus_Vehicle) return Positive;
   function Desviacio_Distancia_Vehicle (C: in Ciutat; Tipus: Tipus_Vehicle) return Integer;
   function Obtenir_Temps_Control_Trafic(C: in Ciutat) return Positive;
   
   -- Funcions de tractament de cruilles i semafors
   procedure Inicialitzar_Cruilla (C: in out Ciutat; Fila, Columna: in Positive;
                                   Color_H, Color_V: in Estat_Semafor);
   procedure Alternar_Semafors (C: in out Ciutat; Fila, Columna: in Positive);
   procedure Travessar_Semafor (C: in out Ciutat; Fila, Columna: in Positive; D: in Direccio);
   function Estat_Actual_Semafor (C: in Ciutat; Fila, Columna: in Positive; D: in Direccio)
                                  return Estat_Semafor;
   function Obtenir_Periode_Semafors (C: in Ciutat) return Positive;
   procedure Establir_Periode_Semafors (C: in out Ciutat; Periode: Positive);
   procedure Modificar_Estat_Semafor (C: in out Ciutat; Fila, Columna: in Positive;
                                      D: in Direccio; Color: in Estat_Semafor);
   function Vehicles_Aturats_Semafor (C: in Ciutat; Fila, Columna: in Positive;
                                      D: in Direccio) return Natural;
   function Control_Trafic_Ciutat (C: in Ciutat) return boolean;

   -- Funcions diverses d'obtencio de coordenades
   procedure Vehicle_En_Cruilla (C: in Ciutat; Pos_X, Pos_Y : in Natural; Pos_Cruilla:
                                 out Coordenada; Dir: in Direccio; Num_Carrer: Positive;
                                 En_Cruilla: out Boolean; Metres_A_Semafor: out Natural;
                                 Meitat_Tam_Vehicle: in Positive);
   function Coordenades_Primera_Cruilla (C: in Ciutat; Dir: in Direccio) return Coordenada;
   function Orientacio_Carrer_H (C: in Ciutat; Carrer: Positive) return Orientacio_Horitzontal;
   function Orientacio_Carrer_V (C: in Ciutat; Carrer: Positive) return Orientacio_Vertical;
   function Tam_Ciutat_H (C: in Ciutat) return Positive;
   function Tam_Ciutat_V (C: in Ciutat) return Positive;

   -- Procediments de interaccio amb l'actualitzacio de la interficie executats
   -- per les tasques Tasca_Vehicle, Tasca_Grua, Tasca_Escombraries i Tasca_Manegador_Cruilla.
   -- No realitzen directament les accions sobre la interficie, sino que notifiquen
   -- a la ciutat que volen realitzar determinades accions, sent la tasca 
   -- Actualitzador_Interficie la encarregada de realitzar centralitzadament totes
   -- les actualitzacions sobre la interficie.
   procedure Moure_Vehicle (C: in out Ciutat; Pos_Actual: in Coordenada; Id: in Positive;
                            Tipus: in Tipus_Vehicle; Grua, Escombraries: in Boolean);
   procedure Vehicle_Avariat (C: in out Ciutat; Pos_Actual: in Coordenada; Id: in Positive;
                            Tipus: in Tipus_Vehicle);
   procedure Canviar_Semafors_Interficie (C: in out Ciutat; Fila, Columna: in Positive;
                                          D: in Direccio; Color: in Estat_Semafor);
   procedure Treure_Vehicle_Garatge (C: in out Ciutat; Grua, Escombraries: Boolean);
   procedure Guardar_Vehicle_Garatge (C: in out Ciutat;  Grua, Escombraries: Boolean);
   procedure Recollir_Vehicle_Avariat (C: in out Ciutat; Pos_Actual: in Coordenada;
                                       Id: Positive; Tipus: Tipus_Vehicle);
   procedure Deixar_Avariat_En_Taller (C: in out Ciutat; Id: Positive; Tipus: Tipus_Vehicle);

   -- Estadistiques
   procedure Actualitzar_Flow (C: in out Ciutat; Flow: Duration);
                           
   -- Procediments d'espera accions i d'actualitzacio de la interficie, executats 
   -- per la tasca Actualitzador_Interficie
   procedure Esperar_Accions_Interficie (C: in out Ciutat; Dades: out Info_Accio);
   procedure Actualitzar_Interficie (Dades: in Info_Accio);
   function Accions_Per_Tractar(C: in Ciutat) return boolean;

private
   type Max_Velocitat is array (Cotxe..Bicicleta) of Positive;
   type Prob_Circular_Recte is array (Cotxe..Bicicleta) of Float;
   type Num_Vehicles is array (Cotxe..Bicicleta) of Natural;
   type Distancia_Mitjana is array (Cotxe..Bicicleta) of Positive;
   type Desviacio_Distancia is array (Cotxe..Bicicleta) of Integer;
   type Cruilles_Ciutat is array (Positive range <>,Positive range <>) of Cruilla;
   type Posicions_Cruilles is array (Positive range <>,Positive range <>) of Coordenades_Cruilla;
   type Sentits_Horitzontals is array (Positive range <>) of Orientacio_Horitzontal;
   type Sentits_Verticals is array (Positive range <>) of Orientacio_Vertical;

   protected type Interficie_Protegida is
      procedure Inicialitzacio;
      procedure Nova_Accio (Dada: in Info_Accio);
      entry Extreure_Accio (Dada: out Info_Accio);
      function Hi_Ha_Accions return boolean;

   private
      Accions_Interficie : Accio;
      Num_Accions : Natural := 0;
   end Interficie_Protegida;

   type Acces_Interficie_Protegida is access Interficie_Protegida;

   type Ciutat(Carrers_H, Carrers_V: Positive) is record
      Num_Carrers_H : Positive := Carrers_H;
      Num_Carrers_V : Positive := Carrers_V;
      Sentits_Carrers_H : Sentits_Horitzontals(1..Carrers_H);
      Sentits_Carrers_V : Sentits_Verticals(1..Carrers_V);
      Num_Cruilles : Positive := Carrers_H * Carrers_V;
      Velocitat : Max_Velocitat;
      Probabilitat : Prob_Circular_Recte;
      Nombre_Inicial : Num_Vehicles;
      Distancia_Vehicle : Distancia_Mitjana;
      Desviacio : Desviacio_Distancia;
      Uts : Positive;
      Separacio_Cruilles : Positive;
      Periode_Escombraries : Positive;
      Minuts_Per_Contenidor : Natural;
      Periode_Semafors : Positive;
      Taula_Cruilles : Cruilles_Ciutat(1..Carrers_H,1..Carrers_V);
      Pos_Cruilles : Posicions_Cruilles(1..Carrers_H,1..Carrers_V);
      Accions : Acces_Interficie_Protegida;
      Coord_Garatge : Coordenada;
      Carrer_Garatge : Positive;
      Direccio_Garatge : Direccio;
      Coord_Taller : Coordenada;
      Carrer_Taller : Positive;
      Direccio_Taller : Direccio;
      Longitud_H_Ciutat : Positive;
      Longitud_V_Ciutat : Positive;
      Amplada_Carrers : Positive;
      Control_Trafic_Activat : Boolean;
      Tamany_Cotxe : Positive;
      Tamany_Camio : Positive;
      Tamany_Bicicleta : Positive;
      Tamany_Ciclomotor : Positive;
      FlowAverage : Duration;
      TempsInici : Time;
      Divisor_Minuts_Controlador : Positive;
   end record;



end Ciutats;



