
-------------------------------------------------------------------------------------------
-- Practica de Enginyeria del Software II / Curs 2003 - 2004                             --
-- Simulador i Controlador de Transit en Temps Real                                      --
--                                                                                       --
-- Funcions_Tasques.adb: Fitxer que conte la implementacio de diversos procediments i    --
--                       funcions que son d'us comu en diverses tasques del modul        --
--                       tasques.adb (tasques vehicle, grua i camio escombraries).       --
-- Realitzat per: Francisco Jose Aguilar Celdran                                         --
--                Sergio Blanco Cuaresma                                                 --
-------------------------------------------------------------------------------------------

with Ada.Text_IO; use Ada.Text_IO;
with Ada.Exceptions; use Ada.Exceptions;
with Ada.Numerics.Discrete_Random;

package body Funcions_Tasques is

   subtype Rang_Aleatoris is Integer range 1..50;
   package Aleatoris is new Ada.Numerics.Discrete_Random(Result_Subtype => Rang_Aleatoris);
   use Aleatoris;   

   ------------------------------------
   -- Posicionament_Aleatori_Inicial --
   ------------------------------------

   procedure Posicionament_Aleatori_Inicial (C: in out Ciutat; Carrers_H, Carrers_V, Carrer_H_Actual,
                                             Carrer_V_Actual,Metres_Cruilles: out Positive;
                                             Pos_Actual: out Coordenada; Id: in Positive;
                                             Tipus: in Tipus_Vehicle) is
      Llavor : Generator;
      Cruilla_Ini_H, Cruilla_Ini_V : Coordenada;
   begin
      Reset(Llavor);
      Cruilla_Ini_H := Coordenades_Primera_Cruilla(C,Horitzontal);
      Cruilla_Ini_V := Coordenades_Primera_Cruilla(C,Vertical);
      Metres_Cruilles := Separacio_Entre_Cruilles(C) + Ample_Carrer(C);
      Carrers_H := Num_Carrers(C,Horitzontal);
      Carrers_V := Num_Carrers(C,Vertical);
      Carrer_H_Actual := (Random(Llavor) mod Carrers_H) + 1;
      Carrer_V_Actual := (Random(Llavor) mod Carrers_V) + 1;
      Pos_Actual.X := Carrer_V_Actual * Metres_Cruilles - Positive(Ample_Carrer(C)/2);
      Pos_Actual.Y := Carrer_H_Actual * Metres_Cruilles - Positive(Ample_Carrer(C)/2);
      Moure_Vehicle(C,Pos_Actual,Id,Tipus,false,false);         
   end Posicionament_Aleatori_Inicial;

   -----------------------------
   -- Actualitzar_Coordenades --
   -----------------------------

   function Actualitzar_Coordenades (C: Ciutat; Direccio_Actual: Direccio; Pos_Ini: Coordenada;
                                     Metres_Desplasats: Natural; Carrer_H_Actual,
                                     Carrer_V_Actual: Positive) return Coordenada is
      Pos_Actual : Coordenada;
   begin 
      case Direccio_Actual is
         when Horitzontal =>
            if Orientacio_Carrer_H(C,Carrer_H_Actual) = Dreta then
               Pos_Actual.X := ((Pos_Ini.X + Metres_Desplasats) mod Tam_Ciutat_H(C));
            else
               Pos_Actual.X := ((Pos_Ini.X - Metres_Desplasats) mod Tam_Ciutat_H(C));
            end if;
            if (Pos_Actual.X = 0) then
               Pos_Actual.X := Tam_Ciutat_H(C);
            end if;
            Pos_Actual.Y := Pos_Ini.Y;
         when Vertical =>
            if Orientacio_Carrer_V(C,Carrer_V_Actual) = Amunt then
               Pos_Actual.Y := ((Pos_Ini.Y - Metres_Desplasats) mod Tam_Ciutat_V(C));
            else
               Pos_Actual.Y := ((Pos_Ini.Y + Metres_Desplasats) mod Tam_Ciutat_V(C));
            end if;
            if (Pos_Actual.Y = 0) then
               Pos_Actual.Y := Tam_Ciutat_V(C);
            end if;
            Pos_Actual.X := Pos_Ini.X;
      end case;
      return Pos_Actual;
   exception
         when E: others => Put_Line("Excepcio en Actualitzar_Coordenades " & Exception_Name(E) & ", causada per: " & Exception_Message(E));
         raise Constraint_Error;
   end Actualitzar_Coordenades;
     
   ------------------------
   -- Actualitzar_Carrer --
   ------------------------

   procedure Actualitzar_Carrer (C: in Ciutat; C_H, C_V: in out Positive;
                                 Direccio_Actual: in Direccio) is
      Num_Carrers_H, Num_Carrers_V : Positive;
      Carrer_H, Carrer_V : Natural;
   begin
      Num_Carrers_H := Num_Carrers(C,Horitzontal);
      Num_Carrers_V := Num_Carrers(C,Vertical);
      if (Direccio_Actual = Horitzontal) then
         if Orientacio_Carrer_H(C,C_H) = Dreta then
            Carrer_V := ((C_V + 1) mod Num_Carrers_V); -- + 1;
         else
            Carrer_V := ((C_V - 1) mod Num_Carrers_V); -- + 1;
         end if;
         if (Carrer_V = 0) then
            Carrer_V := Num_Carrers_V;
         end if;
         C_V := Carrer_V;
      else
         if Orientacio_Carrer_V(C,C_V) = Abaix then
            Carrer_H := ((C_H + 1) mod Num_Carrers_H); -- + 1;
         else
            Carrer_H := ((C_H - 1) mod Num_Carrers_H); -- + 1;
         end if;
         if (Carrer_H = 0) then
            Carrer_H := Num_Carrers_H;
         end if;            
         C_H := Carrer_H;
      end if;
   end Actualitzar_Carrer;

   ---------------------------
   -- Decidir_Direccio_Grua --
   ---------------------------

   procedure Decidir_Direccio_Grua (C: in Ciutat; Pos_Actual: in Coordenada; Desti_X, Desti_Y,
                                    Carrer_H, Carrer_V : in Positive; Direccio_Actual: in out Direccio;
                                    Carrer_Desti: in Positive; Dir_Desti: in Direccio) is

      function Decidir_Segons_Distancia_I_Sentit(Direccio_Dif: Direccio; Diferencia: Integer;
                                                 Sentit_H: Orientacio_Horitzontal;
                                                 Sentit_V: Orientacio_Vertical) return Direccio is
         Direccio_Actual : Direccio;
         Diferencia_X, Diferencia_Y : Integer;
      begin
         if Direccio_Dif = Horitzontal then
            Diferencia_X := Diferencia;
            if Diferencia_X < 0 then                -- Desti a l'esquerra
               if Sentit_H = Esquerra then          -- Carrer amb direccio desitjada
                  Direccio_Actual := Horitzontal;
               else                                 -- Carrer amb direccio oposada
                  Direccio_Actual := Vertical;
               end if;
            else                                    -- Desti a la dreta
               if Sentit_H = Dreta then             -- Carrer amb direccio desitjada
                  Direccio_Actual := Horitzontal;
               else                                 -- Carrer amb direccio oposada
                  Direccio_Actual := Vertical;
               end if;
            end if;  
         else
            Diferencia_Y := Diferencia;
            if Diferencia_Y < 0 then                -- Desti al nord
               if Sentit_V = Amunt then             -- Carrer amb direccio desitjada
                  Direccio_Actual := Vertical;
               else                                 -- Carrer amb direccio oposada
                  Direccio_Actual := Horitzontal;
               end if;
            else                                    -- Desti al sud
               if Sentit_V = Abaix then             -- Carrer amb direccio desitjada
                  Direccio_Actual := Vertical;
               else                                 -- Carrer amb direccio oposada
                  Direccio_Actual := Horitzontal;
               end if;
            end if;               
         end if;
         return Direccio_Actual;
      end Decidir_Segons_Distancia_I_Sentit;
         
   begin
      if Direccio_Actual = Horitzontal and Dir_Desti = Horitzontal and Carrer_H = Carrer_Desti then         
         Direccio_Actual := Horitzontal;
      elsif Direccio_Actual = Vertical and Dir_Desti = Vertical and Carrer_V = Carrer_Desti then
         Direccio_Actual := Vertical;
      else
         if Dir_Desti = Vertical and Direccio_Actual = Vertical then
            Direccio_Actual := Horitzontal;
         elsif Dir_Desti = Horitzontal and Direccio_Actual = Horitzontal then
            Direccio_Actual := Vertical;
         elsif Dir_Desti = Horitzontal and Direccio_Actual = Vertical then
            if Carrer_H = Carrer_Desti then
               Direccio_Actual := Horitzontal;
            else
               Direccio_Actual := Vertical;
            end if;
         elsif Dir_Desti = Vertical and Direccio_Actual = Horitzontal then
            if Carrer_V = Carrer_Desti then
               Direccio_Actual := Vertical;
            else
               Direccio_Actual := Horitzontal;
            end if;
         end if;
      end if;      
   exception
      when E: others => Put_Line("Excepcio en Decidir_Direccio_Grua " & Exception_Name(E) & ", causada per: " & Exception_Message(E));

   end Decidir_Direccio_Grua;

   -------------------------
   -- Carrers_H_V_Immoble --
   -------------------------

   procedure Carrers_H_V_Immoble (C: in Ciutat; Carrer: in Positive; Dir: in Direccio;
                                  Pos: in Coordenada; Carrer_H, Carrer_V: out Positive) is
      C1, C2 : Positive;
   begin
      if Dir = Horitzontal then
         Carrer_H := Carrer;
         Carrers_Entre_Coordenada(C,Pos,C1,C2,Horitzontal);
         if Orientacio_Carrer_H(C,Carrer_H) = Esquerra then
            Carrer_V := C2;
         else
            Carrer_V := C1;
         end if;
      else
         Carrer_V := Carrer;
         Carrers_Entre_Coordenada(C,Pos,C1,C2,Vertical);
         if Orientacio_Carrer_V(C,Carrer_V) = Amunt then
            Carrer_H := C2;
         else
            Carrer_H := C1;
         end if;         
      end if;
   end Carrers_H_V_Immoble;

end Funcions_Tasques;
