
-------------------------------------------------------------------------------------------
-- Practica de Enginyeria del Software II / Curs 2003 - 2004                             --
-- Simulador i Controlador de Transit en Temps Real                                      --
--                                                                                       --
-- Interficievisual.adb: Implementacio de la interficie grafica del simulador per mitja  --
--                       de GTK-ADA. Aquest modul conte els metodes necessaris per       --
--                       crear la interficie i actualizar-hi tots els components.        --
--                       S'empra una implementacio de doble buffer que permet            --
--                       actualizar la interficie periodicament de forma suau.           --
-- Realitzat per: Francisco Jose Aguilar Celdran                                         --
--                Sergio Blanco Cuaresma                                                 --
-------------------------------------------------------------------------------------------

with Glib;              use Glib;
with Glib.Convert;      use Glib.Convert;
with Double_Buffer;     use Double_Buffer;
with Gtk.Window;        use Gtk.Window;
with Gtk.Label;         use Gtk.Label;
with Gtk.Table;         use Gtk.Table;
with Gtk.GEntry;         use Gtk.GEntry;
-----
with Gdk.Font;         use Gdk.Font;
with Gdk.Color;         use Gdk.Color;
with Gdk.Drawable;      use Gdk.Drawable;
with Gdk.Gc;            use Gdk.Gc;
with Gtk.Enums;         use Gtk.Enums;
with Gtk.Handlers;      use Gtk.Handlers;
pragma Elaborate_All (Gtk.Handlers);
with Gtk.Widget;        use Gtk.Widget;
with Gtk.Main;          use Gtk.Main;
with Gtk.Fixed;         use Gtk.Fixed;
pragma Elaborate_All (Gtk.Main);
with Gtk.Button;        use Gtk.Button;
with Gtkada.File_Selection; use Gtkada.File_Selection;
with Gdk.Threads;
with Ada.Real_Time; use Ada.Real_Time;
with Tipus_Comuns; use Tipus_Comuns;
with Ciutats; use Ciutats;
with Tipus_Comuns; use Tipus_Comuns;
with Tasques; use Tasques;

with Ada.Text_IO; use Ada.Text_IO;
with Ada.Integer_Text_IO; use Ada.Integer_Text_IO;

package body InterficieVisual is
   White_Gc : Gdk.Gc.Gdk_GC;
   Black_Gc : Gdk.Gc.Gdk_GC;
   DarkRed_Gc : Gdk.Gc.Gdk_GC;
   Red_Gc   : Gdk.Gc.Gdk_GC;
   Yellow_Gc: Gdk.Gc.Gdk_GC;
   Green_Gc : Gdk.Gc.Gdk_GC;
   Blue_Gc  : Gdk.Gc.Gdk_GC;
   DarkBlue_Gc  : Gdk.Gc.Gdk_GC;
   Gray_Gc  : Gdk.Gc.Gdk_GC;
   Font: Gdk_Font;
   ---
   type Entries is array (0..26) of Gtk_Entry;
   type TSemCruilla is array(0..100) of Boolean;
   
   
    type GUIrecord is record
       WinOptions: Gtk_Window;
       WinCity: Gtk_Window;
       Buffer : Gtk_Double_Buffer;
       -- Estos nombres de variables pueden llevar a confusin:
       -- StreetDistance: Distancia de 1 acera (desde 1 esquina hasta la otra)
       -- CruillesDistance: Distancia entre edificios (calles y cruilles)
       UTS, VStreets, HStreets, StreetDistance, CruillesDistance: Gint;
       HCity, VCity, MetersPixel: Gint;
       TrashSize, SemSize, GarageSize, TallerSize: Gint;
       CarSize, MotoSize, CamioSize, BikeSize: Gint;
       ListSemCruilla: Llista_Sem.Llista;
       CarList, MotoList, CamioList, BikeList: Llista_Vehicle.Llista;
       CamioBrossa, CamioGrua: TVehicle;
       CamioGruaCarregat: Boolean;
       FlowLabel: Gtk_Label;
    end record;
    
   GUI: GUIrecord;
   ---
   type Labels is array (0..26) of Gtk_Label;
   
   type Info_Vehicle is record
      Tipus : Tipus_Vehicle;
      Vel_Max : Positive;
      Duracio : Positive;
      Variacio_Duracio : Natural;
      Nombre_Vehicles: Natural;
   end record;
   type Info_Vehicles is array(Cotxe..Bicicleta) of Info_Vehicle;


   package Handlers is new Gtk.Handlers.Callback (Gtk_Window_Record);
   package Button_Handlers is new Gtk.Handlers.User_Callback (Gtk_Button_Record, Gtk_Window);
   package Button_AcceptOptions is new Gtk.Handlers.User_Callback (Gtk_Button_Record, Entries);
   package Gint_Timeout is new Gtk.Main.Timeout (Integer);
   
   procedure GUI_CityInit;
   --procedure DrawCity;
   procedure GUI_GetOption;
   procedure GUI_InitColors;
   procedure DrawTrash (X, Y: Gint);
   procedure DrawSemaphore (X, Y: Gint; Stop: Boolean);
   procedure DrawGarage (X, Y: Gint);
   procedure DrawTaller (X, Y: Gint);
   procedure DrawCar (X, Y: Gint; Crashed: Boolean);
   procedure DrawCamio (X, Y: Gint; Crashed: Boolean);
   procedure DrawBike (X, Y: Gint; Crashed: Boolean);
   procedure DrawMoto (X, Y: Gint; Crashed: Boolean);
   procedure DrawCamioGrua (X, Y: Gint; Loaded: Boolean);
   procedure DrawCamioBrossa (X, Y: Gint);
   
   ------------- Llista
   procedure Copiar_Sem (S1: in Boolean; S2: out Boolean) is
   begin
      S2 := S1;
   end;
   
   function Comparar_Sem (S1: in Boolean; S2: in Boolean) return boolean is
   begin
      return (S1 = S2);
   end;
   --
   procedure Copiar_Vehicle (V1: in TVehicle; V2: out TVehicle) is
   begin
      V2 := V1;
   end;
   
   function Comparar_Vehicle (V1: in TVehicle; V2: in TVehicle) return boolean is
   begin
      return (V1.Id = V2.Id);
   end;
   -------------
   -----------------------------------------------------------------------------
   -- Termina

   procedure Quit (Win : access Gtk_Window_Record'Class) is
   --    pragma Warnings (Off, Win);
   begin
      Gtk.Main.Gtk_Exit (0);
   end Quit;
   
   -- Termina (para llamarlo desde un boton clickado)
   procedure Quit2 (B: access Gtk_Button_Record'Class; To_Be_Deleted: Gtk_Window) is
   begin
    Gtk.Main.Gtk_Exit (0);
   end Quit2;
   
   -- Debe ser llamado desde un timeout (Gint_Timeout) para que actualice la pantalla.
   -- Si es llamado desde otro lugar, no tendra efecto visible no se pq.
   function RefreshBuffer(id: Integer) return Boolean is
   begin
    Double_Buffer.Draw(GUI.Buffer);
    return true;
   end RefreshBuffer;
   
   task type TascaIniciarExecucio(C: Ciutat_ptr) is
   end TascaIniciarExecucio;
   type AccessTascaIniciarExecucio is access TascaIniciarExecucio;
   
   task body TascaIniciarExecucio is
   begin
      Execucio_Tasques(C.ALL);
   end;
   
   procedure AcceptOptions_cb (B: access Gtk_Button_Record'Class; EntryOption: Entries) is
      Ok: Boolean;
      Periode_Semafors: Positive;
      Periode_Camio_Brossa: Positive;
      Temps_Contenidor: Positive;
      Probabilitat_Recte: Float;
      Vehicles: Info_Vehicles;
      Coord_Cruilla: Coordenades_Cruilla;
      Control_Trafic : Natural;
      Divisor_Temps_Controlador : Positive;
      Controlar : Boolean;
      IniciarExecucioTask: AccessTascaIniciarExecucio;
   begin
     Hide_All(GUI.WinOptions);
     
     GUI.UTS := Gint(Integer'Value(Get_Text(EntryOption(0))));
     GUI.VStreets := Gint(Integer'Value(Get_Text(EntryOption(1))));
     GUI.HStreets := Gint(Integer'Value(Get_Text(EntryOption(2))));

     Crear_Llista(GUI.ListSemCruilla, OK);
     for i in 1..(GUI.VStreets * GUI.HStreets) loop
         Afegir_Element(GUI.ListSemCruilla, true,OK);
     end loop;
     
     Crear_Llista(GUI.CarList, OK);
     Crear_Llista(GUI.MotoList, OK);
     Crear_Llista(GUI.CamioList, OK);
     Crear_Llista(GUI.BikeList, OK);
     GUI.CamioGruaCarregat := false;
     
     Gui.Streetdistance := Gint(Integer'Value(Get_Text(Entryoption(3))));
     Periode_Semafors := Positive(Gint(Integer'Value(Get_Text(Entryoption(23)))));
     Periode_Camio_Brossa := Positive(Gint(Integer'Value(Get_Text(EntryOption(21)))));
     Temps_Contenidor := Positive(Gint(Integer'Value(Get_Text(EntryOption(22)))));
     Gui.Meterspixel := Gint(Integer'Value(Get_Text(Entryoption(24))));
     Control_Trafic := Natural(Gint(Integer'Value(Get_Text(Entryoption(25)))));
     Divisor_Temps_Controlador := Positive(Gint(Integer'Value(Get_Text(Entryoption(26)))));
     if (Control_Trafic > 0) then
        Controlar := True;
     else
        Controlar := False;
     end if;      
     GUI.CruillesDistance := 50;
     GUI.TrashSize := 10;
     GUI.SemSize := 20;
     GUI.GarageSize := 30;
     GUI.TallerSize := 30;
     GUI.CarSize := 25;
     GUI.CamioSize := 35;
     GUI.BikeSize := 20;
     GUI.MotoSize := 25;
     GUI.HCity := (GUI.VStreets+1)*GUI.StreetDistance/GUI.MetersPixel;
     GUI.HCity := GUI.HCity + (GUI.VStreets)*GUI.CruillesDistance/GUI.MetersPixel;  -- Afegim Distancia de les cruilles
     GUI.VCity := (GUI.HStreets+1)*GUI.StreetDistance/GUI.MetersPixel;
     GUI.VCity := GUI.VCity + (GUI.HStreets)*GUI.CruillesDistance/GUI.MetersPixel;  -- Afegim Distancia de les cruilles
     
--     GUI.CamioGrua.X := (GUI.CruillesDistance + GUI.StreetDistance*2)-GUI.GarageSize/2;
--     GUI.CamioGrua.Y := GUI.StreetDistance-GUI.GarageSize/2;
--     GUI.CamioBrossa.X := (GUI.CruillesDistance + GUI.StreetDistance*2)-GUI.GarageSize/2;
--     GUI.CamioBrossa.Y := GUI.StreetDistance-GUI.GarageSize/2;
     
     GUI.CamioGrua.X := (GUI.CruillesDistance + GUI.StreetDistance*2)-GUI.GarageSize;
     GUI.CamioGrua.Y := GUI.StreetDistance-GUI.GarageSize;
     GUI.CamioBrossa.X := (GUI.CruillesDistance + GUI.StreetDistance*2)-GUI.GarageSize;
     GUI.CamioBrossa.Y := GUI.StreetDistance-GUI.GarageSize;
     GUI_CityInit;
     DrawCity;
     --RefreshTimerId := Gint_Timeout.Add (50, RefreshBuffer'Access, 0);
     declare
         --C : Ciutat(Integer(GUI.HStreets),Integer(GUI.VStreets));
         C_ptr: Ciutat_ptr;
         Coordgaratge, Coordtaller: Coordenada;
         Compt: Integer;
        Compt_Horit : Boolean;
         Compt_Vert : Boolean;
      begin
         C_ptr := new Ciutat(Integer(GUI.HStreets),Integer(GUI.VStreets));
         CoordGaratge.X := Integer((GUI.CruillesDistance + GUI.StreetDistance*2)-GUI.GarageSize);
         CoordGaratge.Y := Integer(GUI.StreetDistance + (GUI.CruillesDistance / 2));
         CoordTaller.X := Integer(GUI.CruillesDistance + GUI.StreetDistance);
         Coordtaller.Y := Integer(GUI.StreetDistance + (GUI.CruillesDistance / 2));
         Establir_Parametres_Ciutat(C_ptr.ALL, Positive(Gui.StreetDistance), Positive(Gui.Uts),
                                    Temps_Contenidor, Periode_Semafors, Periode_Camio_Brossa,
                                    Coordgaratge, Coordtaller, 1, 1, Horitzontal, Horitzontal,
                                    --Positive(Gui.Hstreets), Positive(Gui.Vstreets),
                                    Positive(Gui.CruillesDistance), Controlar);
         Establir_Sentits_Carrers(C_Ptr.All,Dreta,Abaix);
         Compt_Vert := True;
         for Fila in 1..Positive(Gui.Hstreets) loop
            Compt_Horit := True;
            for Columna in 1..Positive(Gui.Vstreets) loop
               if Compt_Vert = False then
                  Coord_Cruilla.Parada_H.X := Positive(Gui.Streetdistance) + Positive(Gui.CruillesDistance) + ((Columna - 1) * Positive(Gui.Streetdistance + (Gui.CruillesDistance)));
               else
                  Coord_Cruilla.Parada_H.X := Positive(Gui.Streetdistance) + ((Columna - 1) * Positive(Gui.Streetdistance + (Gui.CruillesDistance)));
               end if;
               Coord_Cruilla.Parada_H.Y := Positive(Gui.StreetDistance) + Positive(Gui.CruillesDistance / 2) + ((Fila - 1) * Positive(Gui.StreetDistance + (Gui.CruillesDistance)));
               Coord_Cruilla.Parada_V.X := Positive(Gui.StreetDistance) + Positive(Gui.CruillesDistance / 2) + (Columna - 1) * Positive(Gui.StreetDistance + (Gui.CruillesDistance));
               if Compt_Horit = False then
                  Coord_Cruilla.Parada_V.Y := Positive(Gui.Streetdistance) + Positive(Gui.CruillesDistance) + ((Fila - 1) * Positive(Gui.Streetdistance + (Gui.CruillesDistance)));
                  Compt_Horit := True;
               else
                  Coord_Cruilla.Parada_V.Y := Positive(Gui.Streetdistance) + ((Fila - 1) * Positive(Gui.Streetdistance + (Gui.CruillesDistance)));
                  Compt_Horit := False;
               end if;
               Establir_Coordenades_Cruilla(C_Ptr.All,Fila,Columna,Coord_Cruilla);
               
            end loop;
            if Compt_Vert = True then
               Compt_Vert := False;
            else
               Compt_Vert := True;
            end if;
         end loop;
         Probabilitat_Recte := Float'Value(Get_Text(Entryoption(12)));
         Compt := 0;
         for Tipus in Cotxe..Bicicleta loop
            Vehicles(Tipus).Tipus := Tipus;
            Vehicles(Tipus).Vel_Max := Positive(Gint(Integer'Value(Get_Text(Entryoption(4 + Compt)))));
            Vehicles(Tipus).Duracio := Positive(Gint(Integer'Value(Get_Text(Entryoption(13 + 2 * Compt)))));
            Vehicles(Tipus).Variacio_Duracio := Natural(Gint(Integer'Value(Get_Text(Entryoption(14 + 2 * Compt)))));
            Vehicles(Tipus).Nombre_Vehicles := Natural(Gint(Integer'Value(Get_Text(Entryoption(8 + Compt)))));
            Establir_Parametres_Vehicle(C_ptr.ALL,Tipus,Vehicles(Tipus).Vel_Max,Vehicles(Tipus).Duracio,
                                        Vehicles(Tipus).Variacio_Duracio,Probabilitat_Recte,
                                        Vehicles(Tipus).Nombre_Vehicles);
            Compt := Compt + 1;
         end loop;
         Establir_Tamany_Vehicle(C_Ptr.All,Positive(Gui.CarSize)/2,Cotxe);
         Establir_Tamany_Vehicle(C_Ptr.All,Positive(Gui.CamioSize)/2,Camio);
         Establir_Tamany_Vehicle(C_Ptr.All,Positive(Gui.MotoSize)/2,Ciclomotor);
         Establir_Tamany_Vehicle(C_Ptr.All,Positive(Gui.BikeSize)/2,Bicicleta);
	 Establir_Temps_Control_Trafic(C_Ptr.All,Divisor_Temps_Controlador);
         IniciarExecucioTask := new TascaIniciarExecucio(C_ptr);
      end;
   end;
   -----------------------------------------------------------------------------
   
   -- Converts from ISO-8859-15 to UTF-8
   function utf8(text: String) return String is
   begin
      return Locale_To_UTF8(text);
   end;
   
   -- Iniciem interficie
   procedure GUI_Start is
   begin
      -- Opcions
      GUI_GetOption;
   end;
   
   procedure GUI_GetOption is
      Table: Gtk_Table;
      Button : Gtk_Button;
      Label: Labels;
      EntryOption: Entries;
   begin
     
      Gtk_New (GUI.WinOptions, Window_Toplevel);
      Set_Title (GUI.WinOptions, utf8("Opcions de simulaci"));
      Set_Policy (GUI.WinOptions, False, true, true); -- Shrink, Grow, Auto
      --Set_USize(GUI.WinOptions, (GUI.Width*2)+20, (GUI.Height*2)+42);
      -- Si cerramos ventana, que acabe la aplicacin
      Handlers.Connect (GUI.WinOptions, "destroy", Handlers.To_Marshaller (Quit'Access));
      
      -- Creamos contenedor Fijo, que se pueden aadir todos los widget que queramos
      -- en cualquier posicion
      Gtk_New(Table, 2, 28, false);
      Add (GUI.WinOptions, Table);
      
      Gtk_New(Label(0), utf8("UTS"));
      Gtk_New(Label(1), utf8("Carrers verticals"));
      Gtk_New(Label(2), utf8("Carrers horitzontals"));
      Gtk_New(Label(3), utf8("Distancia entre crulles (metres)"));
      Gtk_New(Label(4), utf8("Velocitat mxima dels cotxes"));
      Gtk_New(Label(5), utf8("Velocitat mxima dels ciclomotors"));
      Gtk_New(Label(6), utf8("Velocitat mxima dels camions"));
      Gtk_New(Label(7), utf8("Velocitat mxima de les bicicletes"));
      Gtk_New(Label(8), utf8("Nombre de cotxes"));
      Gtk_New(Label(9), utf8("Nombre de ciclomotors"));
      Gtk_New(Label(10), utf8("Nombre de camions"));
      Gtk_New(Label(11), utf8("Nombre de bicicletes"));
      Gtk_New(Label(12), utf8("Probabilitat de continuar recte"));
      Gtk_New(Label(13), utf8("Distncia mitjana abans d'averia del cotxe"));
      Gtk_New(Label(14), utf8("Desviaci estandard distncia anterior"));
      Gtk_New(Label(15), utf8("Distncia mitjana abans d'averia del ciclomotor"));
      Gtk_New(Label(16), utf8("Desviaci estandard distncia anterior"));
      Gtk_New(Label(17), utf8("Distncia mitjana abans d'averia del cami"));
      Gtk_New(Label(18), utf8("Desviaci estandard distncia anterior"));
      Gtk_New(Label(19), utf8("Distncia mitjana abans d'averia de la bicicleta"));
      Gtk_New(Label(20), utf8("Desviaci estandard distncia anterior"));
      Gtk_New(Label(21), utf8("Periodicitat cami d'escombraries (hores)"));
      Gtk_New(Label(22), utf8("Temps recollida brosa del cami d'escombraries (min)"));
      Gtk_New(Label(23), Utf8("Periodicitat dels semafors (min)"));
      Gtk_New(Label(24), utf8("Metres per pixel"));
      Gtk_New(Label(25), utf8("Control de trafic activat (1) o desactivat (0)"));
      Gtk_New(Label(26), utf8("Controlar trafic cada (Periode_Semafor / X) minuts. X s:"));
      
      for i in 0..26 loop
         Set_Justify(Label(i), Justify_Right);
         Attach_Defaults(Table, Label(i), 0, 1, Guint(i), Guint(i+1));
      end loop;
      
      for i in 0..26 loop
         Gtk_New(EntryOption(i));
      end loop;
      
      Set_Text(EntryOption(0), utf8("20"));
      Set_Text(EntryOption(1), utf8("2"));
      Set_Text(EntryOption(2), utf8("2"));
      Set_Text(EntryOption(3), utf8("100"));
      Set_Text(EntryOption(4), utf8("40"));
      Set_Text(EntryOption(5), utf8("25"));
      Set_Text(EntryOption(6), utf8("30"));
      Set_Text(EntryOption(7), utf8("15"));
      Set_Text(EntryOption(8), utf8("1"));
      Set_Text(EntryOption(9), utf8("0"));
      Set_Text(EntryOption(10), utf8("0"));
      Set_Text(EntryOption(11), utf8("0"));
      Set_Text(EntryOption(12), utf8("0.5"));
      Set_Text(EntryOption(13), utf8("5000"));
      Set_Text(EntryOption(14), utf8("100"));
      Set_Text(EntryOption(15), utf8("3000"));
      Set_Text(EntryOption(16), utf8("100"));
      Set_Text(EntryOption(17), utf8("9000"));
      Set_Text(EntryOption(18), utf8("100"));
      Set_Text(EntryOption(19), utf8("1000"));
      Set_Text(EntryOption(20), utf8("100"));
      Set_Text(EntryOption(21), utf8("1"));
      Set_Text(EntryOption(22), utf8("1"));
      Set_Text(Entryoption(23), Utf8("1"));
      Set_Text(Entryoption(24), Utf8("1"));
      Set_Text(EntryOption(25), utf8("0"));
      Set_Text(EntryOption(26), utf8("5"));
      

      for i in 0..26 loop
         Attach_Defaults(Table, EntryOption(i), 1, 2, Guint(i), Guint(i+1));
      end loop;

      -- Creacin de todos los botones
      Gtk_New (Button, "Acceptar");
      Set_Relief (Button, Relief_Normal);
      Attach_Defaults(Table, Button, 0, 2, 27, 28);
      
      Button_AcceptOptions.Connect (Button, "clicked", Button_AcceptOptions.To_Marshaller (AcceptOptions_cb'Access), EntryOption);
      
      -- Mostramos todo
      Show_All (GUI.WinOptions);
   end;
      
   procedure GUI_CityInit is
      Table, TableEstado: Gtk_Table;
      --Fixed  : Gtk_Fixed;
      Button : Gtk_Button;
      FlowLabelText: Gtk_Label;
      --Pixmap : Gdk_Drawable;
   begin
      
      Gtk_New (GUI.WinCity, Window_Toplevel);
      Set_Title (GUI.WinCity, utf8("Simulaci"));
      Set_Policy (GUI.WinCity, False, true, true); -- Shrink, Grow, Auto
      --Set_USize(GUI.WinCity, (GUI.Width*2)+20, (GUI.Height*2)+42);
      -- Si cerramos ventana, que acabe la aplicacin
      Handlers.Connect (GUI.WinCity, "destroy", Handlers.To_Marshaller (Quit'Access));
      
      -- Creamos contenedor Fijo, que se pueden aadir todos los widget que queramos
      -- en cualquier posicion
      Gtk_New(Table, 2, 2, false);
      Add (GUI.WinCity, Table);
      
      -- Nuevo DobleBuffer (el solo se encarga de hacer doble buffering)
      Gtk_New (GUI.Buffer);
      Set_USize (GUI.Buffer, Gint(GUI.HCity), Gint(GUI.VCity));
      
      Attach_Defaults(Table, GUI.Buffer, 0, 1, 0, 1);
      
      -- Creamos etiquetas de estado de fluidez
      Gtk_New(TableEstado, 2, 1, false);
      Attach_Defaults(Table, TableEstado, 0, 1, 1, 2);
      
      Gtk_New(FlowLabelText, utf8("Fluidesa del trnsit: "));
      Attach_Defaults(TableEstado, FlowLabelText, 0, 1, 0, 1);
      
      Gtk_New(GUI.FlowLabel, utf8("-"));
      Attach_Defaults(TableEstado, GUI.FlowLabel, 1, 2, 0, 1);
      
      -- Creacin de todos los botones
      Gtk_New (Button, "Sortir");
      Set_Relief (Button, Relief_Normal);
      Attach_Defaults(Table, Button, 1, 2, 1, 2);
      
      Button_Handlers.Connect (Button, "clicked", Button_Handlers.To_Marshaller (Quit2'Access),GUI.WinCity);
      
      -- Mostramos todo
      Show_All (GUI.WinCity);
      Gui_Initcolors;
   end;
   
   procedure GUI_InitColors is
      Color: Gdk_Color;
   begin 
      -- Cargamos la fuente
      --Load (Font, "-adobe-helvetica-bold-r-normal--14-140-75-75-p-78-iso8859-1");
      --Load (Font, "-adobe-helvetica-bold-r-normal--20-140-100-100-p-105-iso8859-1");
      
      --Load (Font, "-adobe-helvetica-bold-r-normal--34------iso8859-1"); -- ERROR DE EJECUCION AQUI!!!
      Load (Font, "-adobe-helvetica-bold-r-normal--14-140-75-75-p-78-iso8859-1");


      --  Generacin de los colores
      Color := Gdk.Color.Parse ("Gray"); --"Black");
      Gdk.Color.Alloc (Gtk.Widget.Get_Default_Colormap, Color);
      Gdk.Gc.Gdk_New (Black_Gc, Get_Window (GUI.Buffer));
      Gdk.Gc.Set_Foreground (Black_Gc, Color);
      
      Color := Gdk.Color.Parse ("White");
      Gdk.Color.Alloc (Gtk.Widget.Get_Default_Colormap, Color);
      Gdk.Gc.Gdk_New (White_Gc, Get_Window (GUI.Buffer));
      Gdk.Gc.Set_Foreground (White_Gc, Color);
      
      Color := Gdk.Color.Parse ("Red");
      Gdk.Color.Alloc (Gtk.Widget.Get_Default_Colormap, Color);
      Gdk.Gc.Gdk_New (Red_Gc, Get_Window (GUI.Buffer));
      Gdk.Gc.Set_Foreground (Red_Gc, Color);
      
      Color := Gdk.Color.Parse ("#DD0000");
      Gdk.Color.Alloc (Gtk.Widget.Get_Default_Colormap, Color);
      Gdk.Gc.Gdk_New (DarkRed_Gc, Get_Window (GUI.Buffer));
      Gdk.Gc.Set_Foreground (DarkRed_Gc, Color);
      
      Color := Gdk.Color.Parse ("WHITE"); --("#F7DE0D");
      Gdk.Color.Alloc (Gtk.Widget.Get_Default_Colormap, Color);
      Gdk.Gc.Gdk_New (Yellow_Gc, Get_Window (GUI.Buffer));
      Gdk.Gc.Set_Foreground (Yellow_Gc, Color);
      
      Color := Gdk.Color.Parse ("#00AA00");
      Gdk.Color.Alloc (Gtk.Widget.Get_Default_Colormap, Color);
      Gdk.Gc.Gdk_New (Green_Gc, Get_Window (GUI.Buffer));
      Gdk.Gc.Set_Foreground (Green_Gc, Color);

      Color := Gdk.Color.Parse ("#5555FF");
      Gdk.Color.Alloc (Gtk.Widget.Get_Default_Colormap, Color);
      Gdk.Gc.Gdk_New (Blue_Gc, Get_Window (GUI.Buffer));
      Gdk.Gc.Set_Foreground (Blue_Gc, Color);
      
      Color := Gdk.Color.Parse ("#000077");
      Gdk.Color.Alloc (Gtk.Widget.Get_Default_Colormap, Color);
      Gdk.Gc.Gdk_New (DarkBlue_Gc, Get_Window (GUI.Buffer));
      Gdk.Gc.Set_Foreground (DarkBlue_Gc, Color);
      
      Color := Gdk.Color.Parse ("#996666"); --("Brown"); --("Gray");
      Gdk.Color.Alloc (Gtk.Widget.Get_Default_Colormap, Color);
      Gdk.Gc.Gdk_New (Gray_Gc, Get_Window (GUI.Buffer));
      Gdk.Gc.Set_Foreground (Gray_Gc, Color);
      
   end;
      

   procedure DrawCity is
      Pixmap : Gdk_Drawable;
      Pos, PosX, PosY: gint;
      Vehicle: TVehicle;
   begin
      Pixmap:=Get_Pixmap (Gtk_Double_Buffer (GUI.Buffer));
      
      -- ClearScreen
      Draw_Rectangle(Pixmap, Gray_Gc, True, 0, 0, Gint(GUI.HCity), Gint(GUI.VCity));
      -- Recuadro
      Draw_Rectangle(Pixmap, Black_Gc, False, 0, 0, Gint(GUI.HCity)-1, Gint(GUI.VCity)-1);
      
      -- Carrers
      for i in 0..GUI.VStreets loop
         Pos := Gint((((i+1)*GUI.StreetDistance + i*GUI.CruillesDistance))/GUI.MetersPixel);
         Draw_Rectangle(Pixmap, Black_Gc, true, Pos, 0, Gint(GUI.CruillesDistance/GUI.MetersPixel), Gint(GUI.VCity)-1);
      end loop;
      
      for i in 0..GUI.HStreets loop
         Pos := Gint((((i+1)*GUI.StreetDistance + i*GUI.CruillesDistance))/GUI.MetersPixel);
         Draw_Rectangle(Pixmap, Black_Gc, true, 0, Pos, Gint(GUI.HCity)-1, Gint(GUI.CruillesDistance/GUI.MetersPixel));
      end loop;

      -- Linia carretera
      for i in 0..GUI.VStreets loop
         Pos := Gint((((i+1)*GUI.StreetDistance + i*GUI.CruillesDistance))/GUI.MetersPixel);      
         Draw_Line (Pixmap, Yellow_Gc, Pos+Gint((GUI.CruillesDistance/2)/GUI.MetersPixel), 0, Pos+Gint((GUI.CruillesDistance/2)/GUI.MetersPixel), Gint(GUI.VCity));
      end loop;
      
      for i in 0..GUI.HStreets loop
         Pos := Gint((((i+1)*GUI.StreetDistance + i*GUI.CruillesDistance))/GUI.MetersPixel);      
         Draw_Line (Pixmap, Yellow_Gc, 0, Pos+Gint((GUI.CruillesDistance/2)/GUI.MetersPixel), Gint(GUI.HCity), Pos+Gint((GUI.CruillesDistance/2)/GUI.MetersPixel));
      end loop;
      
      -- Brossa als carrers horitzontals, a la dreta segons el sentit (altern)
      for i in 1..GUI.HStreets loop
         if (i mod 2) /= 0 then
            PosY := ((i*GUI.StreetDistance + i*GUI.CruillesDistance))/GUI.MetersPixel;
            for j in 0..GUI.VStreets loop
               PosX := ((j+1)*GUI.StreetDistance + j*GUI.CruillesDistance - GUI.StreetDistance/2 - GUI.TrashSize/2)/GUI.MetersPixel;
               DrawTrash(PosX, PosY);
            end loop;
         else
            PosY := (((i)*GUI.StreetDistance + (i-1)*GUI.CruillesDistance - GUI.TrashSize))/GUI.MetersPixel;
            for j in 0..GUI.VStreets loop
            --for j in 0..Gint(0) loop
               PosX := ((j+1)*GUI.StreetDistance + j*GUI.CruillesDistance - GUI.StreetDistance/2 - GUI.TrashSize/2)/GUI.MetersPixel;
               DrawTrash(PosX, PosY);
            end loop;
         end if;
      end loop;
      
      -- Brossa als carrers verticals, a la dreta segons el sentit (altern)
      for i in 1..GUI.VStreets loop
         if (i mod 2) /= 0 then
            PosX := (((i)*GUI.StreetDistance + (i-1)*GUI.CruillesDistance - GUI.TrashSize))/GUI.MetersPixel;
            for j in 0..GUI.HStreets loop
               PosY := ((j+1)*GUI.StreetDistance + j*GUI.CruillesDistance - GUI.StreetDistance/2 - GUI.TrashSize/2)/GUI.MetersPixel;
               DrawTrash(PosX, PosY);
            end loop;
         else
            PosX := ((i*GUI.StreetDistance + i*GUI.CruillesDistance))/GUI.MetersPixel;
            for j in 0..GUI.HStreets loop
               PosY := ((j+1)*GUI.StreetDistance + j*GUI.CruillesDistance - GUI.StreetDistance/2 - GUI.TrashSize/2)/GUI.MetersPixel;
               DrawTrash(PosX, PosY);
            end loop;
         end if;
         
      end loop;
      
      DrawGarage(((GUI.CruillesDistance + GUI.StreetDistance*2)-GUI.GarageSize)/GUI.MetersPixel, (GUI.StreetDistance-GUI.GarageSize)/GUI.MetersPixel);
      DrawTaller((GUI.CruillesDistance + GUI.StreetDistance)/GUI.MetersPixel, (GUI.StreetDistance-GUI.GarageSize)/GUI.MetersPixel);
      
      -- Dibuixar cotxes
      Establir_PDI_Inici(GUI.CarList);
      while Queden_Elements(GUI.CarList) loop
         Vehicle := Seguent_Element(GUI.CarList);
         DrawCar(Vehicle.X / GUI.MetersPixel, Vehicle.Y / GUI.MetersPixel, Vehicle.Avariat);
         Augmentar_Posicio_PDI(GUI.CarList);
      end loop;
      
      -- Dibuixar camions
      Establir_PDI_Inici(GUI.CamioList);
      while Queden_Elements(GUI.CamioList) loop
         Vehicle := Seguent_Element(GUI.CamioList);
         DrawCamio(Vehicle.X / GUI.MetersPixel, Vehicle.Y / GUI.MetersPixel, Vehicle.Avariat);
         Augmentar_Posicio_PDI(GUI.CamioList);
      end loop;
      
      -- Dibuixar bicicletes
      Establir_PDI_Inici(GUI.BikeList);
      while Queden_Elements(GUI.BikeList) loop
         Vehicle := Seguent_Element(GUI.BikeList);
         DrawBike(Vehicle.X / GUI.MetersPixel, Vehicle.Y / GUI.MetersPixel, Vehicle.Avariat);
         Augmentar_Posicio_PDI(GUI.BikeList);
      end loop;
      
      -- Dibuixar motos
      Establir_PDI_Inici(GUI.MotoList);
      while Queden_Elements(GUI.MotoList) loop
         Vehicle := Seguent_Element(GUI.MotoList);
         DrawMoto(Vehicle.X / GUI.MetersPixel, Vehicle.Y / GUI.MetersPixel, Vehicle.Avariat);
         Augmentar_Posicio_PDI(GUI.MotoList);
      end loop;
     
      -- Camions especials
      DrawCamioGrua(GUI.CamioGrua.X / GUI.MetersPixel, GUI.CamioGrua.Y / GUI.MetersPixel, GUI.CamioGruaCarregat);
      DrawCamioBrossa(GUI.CamioBrossa.X / GUI.MetersPixel, GUI.CamioBrossa.Y / GUI.MetersPixel);
      
      -- Semafors Horitzontals
      Establir_PDI_Inici(GUI.ListSemCruilla);
      for i in 1..GUI.HStreets loop
         if (i mod 2) /= 0 then
            PosY := ((i*GUI.StreetDistance + i*GUI.CruillesDistance - GUI.SemSize/2))/GUI.MetersPixel;
            for j in 0..GUI.VStreets-1 loop
               PosX := ((j+1)*GUI.StreetDistance + j*GUI.CruillesDistance - GUI.SemSize *3/2)/GUI.MetersPixel;
               DrawSemaphore(PosX, PosY, not Seguent_Element(GUI.ListSemCruilla));
               Augmentar_Posicio_PDI(GUI.ListSemCruilla);
            end loop;
         else
            PosY := (((i)*GUI.StreetDistance + (i-1)*GUI.CruillesDistance  - GUI.SemSize/2))/GUI.MetersPixel;
            for j in 0..GUI.VStreets-1 loop
               PosX := ((j+1)*GUI.StreetDistance + (j+1)*GUI.CruillesDistance + GUI.SemSize/2)/GUI.MetersPixel;
               DrawSemaphore(PosX, PosY, not Seguent_Element(GUI.ListSemCruilla));
               Augmentar_Posicio_PDI(GUI.ListSemCruilla);
            end loop;
         end if;
      end loop;
      
      -- Semafors verticals
      Establir_PDI_Inici(GUI.ListSemCruilla);
      for i in 0..GUI.HStreets-1 loop
         for j in 0..GUI.VStreets-1 loop
            if (j mod 2) /= 0 then
               PosY := ((i+1)*GUI.StreetDistance + (i+1)*GUI.CruillesDistance + GUI.SemSize *1/2)/GUI.MetersPixel;
               PosX := (((j+1)*GUI.StreetDistance + (j+1)*GUI.CruillesDistance - GUI.SemSize/2))/GUI.MetersPixel;
               DrawSemaphore(PosX, PosY, Seguent_Element(GUI.ListSemCruilla));
               Augmentar_Posicio_PDI(GUI.ListSemCruilla);
            else
               PosY := ((i+1)*GUI.StreetDistance + i*GUI.CruillesDistance - GUI.SemSize *3/2)/GUI.MetersPixel;
               PosX := (((j+1)*GUI.StreetDistance + j*GUI.CruillesDistance - GUI.SemSize/2))/GUI.MetersPixel;
               DrawSemaphore(PosX, PosY, Seguent_Element(GUI.ListSemCruilla));
               Augmentar_Posicio_PDI(GUI.ListSemCruilla);
            end if;
         end loop;
      end loop;
      

      -- Copiamos el buffer secundario al principal (actualizamos pantalla)
      Double_Buffer.Draw(GUI.Buffer);
   end DrawCity;

   procedure DrawTrash (X, Y: Gint) is
       Pixmap : Gdk_Drawable;
   begin
       Pixmap:=Get_Pixmap (Gtk_Double_Buffer (GUI.Buffer));
       Draw_Rectangle(Pixmap, Blue_Gc, True, X, Y, GUI.TrashSize/GUI.MetersPixel, GUI.TrashSize/GUI.MetersPixel);
   end DrawTrash;
   
   procedure DrawSemaphore (X, Y: Gint; Stop: Boolean) is
       Pixmap : Gdk_Drawable;
   begin
      Pixmap:=Get_Pixmap (Gtk_Double_Buffer (GUI.Buffer));
      if Stop then
         Draw_Arc(Pixmap, Red_Gc, True, X, Y, GUI.SemSize/GUI.MetersPixel, GUI.SemSize/GUI.MetersPixel, 0, 360*64);
      else 
         Draw_Arc(Pixmap, Green_Gc, True, X, Y, GUI.SemSize/GUI.MetersPixel, GUI.SemSize/GUI.MetersPixel, 0, 360*64);
      end if;
   end DrawSemaphore;
   
   procedure DrawGarage (X, Y: Gint) is
       Pixmap : Gdk_Drawable;
   begin
       Pixmap:=Get_Pixmap (Gtk_Double_Buffer (GUI.Buffer));
       Draw_Rectangle(Pixmap, DarkRed_Gc, True, X, Y, GUI.GarageSize/GUI.MetersPixel, GUI.GarageSize/GUI.MetersPixel);
   end DrawGarage;
   
   procedure DrawTaller (X, Y: Gint) is
       Pixmap : Gdk_Drawable;
   begin
       Pixmap:=Get_Pixmap (Gtk_Double_Buffer (GUI.Buffer));
       Draw_Rectangle(Pixmap, White_Gc, True, X, Y, GUI.TallerSize/GUI.MetersPixel, GUI.TallerSize/GUI.MetersPixel);
   end DrawTaller;

   procedure DrawCar (X, Y: Gint; Crashed: Boolean) is
       Pixmap : Gdk_Drawable;
   begin
       Pixmap:=Get_Pixmap (Gtk_Double_Buffer (GUI.Buffer));
       Draw_Rectangle(Pixmap, White_Gc, True, X-(GUI.CarSize/(2*GUI.MetersPixel)), Y-(GUI.CarSize/(2*GUI.MetersPixel)), GUI.CarSize/GUI.MetersPixel, GUI.CarSize/GUI.MetersPixel);
       Draw_Arc(Pixmap, Red_Gc, True, X  - ((GUI.CarSize/4)/GUI.MetersPixel), Y - ((GUI.CarSize/4)/GUI.MetersPixel), (GUI.CarSize/2)/GUI.MetersPixel, (GUI.CarSize/2)/GUI.MetersPixel, 0, 360*64);
       if (Crashed) then
           Draw_Text(Pixmap, Font, Red_Gc, X + (((GUI.CarSize)/3 )/GUI.MetersPixel), Y + (((GUI.CarSize)/3 )/GUI.MetersPixel), utf8("X"));
       end if;
   end DrawCar;
   
   procedure DrawCamio (X, Y: Gint; Crashed: Boolean) is
       Pixmap : Gdk_Drawable;
   begin
       Pixmap:=Get_Pixmap (Gtk_Double_Buffer (GUI.Buffer));
       Draw_Rectangle(Pixmap, White_Gc, True, X - (GUI.CamioSize/(2*GUI.MetersPixel)), Y-(GUI.CamioSize/(2*GUI.MetersPixel)), GUI.CamioSize/GUI.MetersPixel, GUI.CamioSize/GUI.MetersPixel);
       Draw_Arc(Pixmap, Blue_Gc, True, X - ((GUI.CamioSize/4)/GUI.MetersPixel), Y - ((GUI.CamioSize/4)/GUI.MetersPixel), (GUI.CamioSize/2)/GUI.MetersPixel, (GUI.CamioSize/2)/GUI.MetersPixel, 0, 360*64);
       if (Crashed) then
           Draw_Text(Pixmap, Font, Red_Gc, X - (GUI.CamioSize/(2*GUI.MetersPixel)), Y + (((GUI.CamioSize)/2 )/GUI.MetersPixel), utf8("X"));
       end if;
   end DrawCamio;
   
   procedure DrawBike (X, Y: Gint; Crashed: Boolean) is
       Pixmap : Gdk_Drawable;
   begin
       Pixmap:=Get_Pixmap (Gtk_Double_Buffer (GUI.Buffer));
       Draw_Rectangle(Pixmap, DarkBlue_Gc, True, X-(GUI.BikeSize/(2*GUI.MetersPixel)), Y-(GUI.BikeSize/(2*GUI.MetersPixel)), GUI.BikeSize/GUI.MetersPixel, GUI.BikeSize/GUI.MetersPixel);
       Draw_Arc(Pixmap, Yellow_Gc, True, X - ((GUI.BikeSize/2)/GUI.MetersPixel), Y - ((GUI.BikeSize/4)/GUI.MetersPixel), (GUI.BikeSize/2)/GUI.MetersPixel, (GUI.BikeSize/2)/GUI.MetersPixel, 0, 360*64);
       Draw_Arc(Pixmap, Yellow_Gc, True, X, Y - ((GUI.BikeSize/4)/GUI.MetersPixel), (GUI.BikeSize/2)/GUI.MetersPixel, (GUI.BikeSize/2)/GUI.MetersPixel, 0, 360*64);
       if (Crashed) then
           Draw_Text(Pixmap, Font, Red_Gc, X, Y + (((GUI.BikeSize)/4 )/GUI.MetersPixel), utf8("X"));
       end if;
   end DrawBike;
   
   procedure DrawMoto (X, Y: Gint; Crashed: Boolean) is
       Pixmap : Gdk_Drawable;
   begin
       Pixmap:=Get_Pixmap (Gtk_Double_Buffer (GUI.Buffer));
       Draw_Rectangle(Pixmap, DarkBlue_Gc, True, X-(GUI.MotoSize/(2*GUI.MetersPixel)), Y-(GUI.MotoSize/(2*GUI.MetersPixel)), GUI.MotoSize/GUI.MetersPixel, GUI.MotoSize/GUI.MetersPixel);
       Draw_Arc(Pixmap, Gray_Gc, True, X - (GUI.MotoSize/4)/GUI.MetersPixel, Y - ((GUI.MotoSize/2)/GUI.MetersPixel), (GUI.MotoSize/2)/GUI.MetersPixel, (GUI.MotoSize/2)/GUI.MetersPixel, 0*64, 280*64);
       Draw_Arc(Pixmap, Gray_Gc, True, X - (GUI.MotoSize/4)/GUI.MetersPixel, Y, (GUI.MotoSize/2)/GUI.MetersPixel, (GUI.MotoSize/2)/GUI.MetersPixel, 0*64, 280*64);
       if (Crashed) then
           Draw_Text(Pixmap, Font, Red_Gc, X + (((GUI.MotoSize)/4 )/GUI.MetersPixel), Y + (((GUI.MotoSize)/4 )/GUI.MetersPixel), utf8("X"));
       end if;
   end DrawMoto;
   
   
   procedure DrawCamioGrua (X, Y: Gint; Loaded: Boolean) is
       Pixmap : Gdk_Drawable;
   begin
       Pixmap:=Get_Pixmap (Gtk_Double_Buffer (GUI.Buffer));
       Draw_Rectangle(Pixmap, DarkRed_Gc, True, X-(GUI.CamioSize/(2*GUI.MetersPixel)), Y-(GUI.CamioSize/(2*GUI.MetersPixel)), GUI.CamioSize/GUI.MetersPixel, GUI.CamioSize/GUI.MetersPixel);
       
       if (Loaded) then
           Draw_Text(Pixmap, Font, Black_Gc, X- (((GUI.CamioSize)/3 )/GUI.MetersPixel), Y, utf8("[]"));
       end if;
       Draw_Arc(Pixmap, Blue_Gc, True, X, Y - ((GUI.CamioSize/4)/GUI.MetersPixel), (GUI.CamioSize/2)/GUI.MetersPixel, (GUI.CamioSize/2)/GUI.MetersPixel, 0, 360*64);
   end DrawCamioGrua;
   
   procedure DrawCamioBrossa (X, Y: Gint) is
       Pixmap : Gdk_Drawable;
   begin
       Pixmap:=Get_Pixmap (Gtk_Double_Buffer (GUI.Buffer));
       Draw_Rectangle(Pixmap, Green_Gc, True, X-(GUI.CamioSize/(2*GUI.MetersPixel)), Y-(GUI.CamioSize/(2*GUI.MetersPixel)), GUI.CamioSize/GUI.MetersPixel, GUI.CamioSize/GUI.MetersPixel);
       Draw_Arc(Pixmap, Blue_Gc, True, X - ((GUI.CamioSize/4)/GUI.MetersPixel), Y - ((GUI.CamioSize/4)/GUI.MetersPixel), (GUI.CamioSize/2)/GUI.MetersPixel, (GUI.CamioSize/2)/GUI.MetersPixel, 0, 360*64);
   end DrawCamioBrossa;
   
   -- X, Y comencen per 0
   procedure ChangeSemaphore (X, Y: Gint; Stop: Boolean) is
   begin
      Establir_PDI_Inici(GUI.ListSemCruilla);
      -- Ens desplaem fins la columna desitjada
      for i in 0..(X * GUI.VStreets-1) loop
         Augmentar_Posicio_PDI(GUI.ListSemCruilla);
      end loop;
      -- Ens desplacem fins la fila desitjada
      for i in 0..Y-1 loop
         Augmentar_Posicio_PDI(GUI.ListSemCruilla);
      end loop;
      
      Modificar_Seguent_Element(GUI.ListSemCruilla, Stop);
   end;
  
   procedure AddVehicle(V: TVehicle; Tipus: Tipus_Vehicle) is
      OK: Boolean;
   begin
      if (Tipus = Cotxe) then
         Afegir_Element (GUI.CarList, V, OK);
      elsif (Tipus = Camio) then
         Afegir_Element (GUI.CamioList, V, OK);
      elsif (Tipus = Ciclomotor) then
         Afegir_Element (GUI.MotoList, V, OK);
      else
         Afegir_Element (GUI.BikeList, V, OK);
      end if;
   end; 
   
   
   -- Mou un cotxe, treu de la llista l'antic i afegeix el nou
   -- en cas de ser el primer moviment (no esta a la llista),
   -- l'afegeix.
   procedure MoveVehicle(V: TVehicle; Tipus: Tipus_Vehicle) is
      OK: Boolean;
   begin
      -- Reemplaar element a la llista
      if (Tipus = Cotxe) then
         Esborrar_Element (GUI.CarList, V, OK);
         Afegir_Element (GUI.CarList, V, OK);
      elsif (Tipus = Camio) then
         Esborrar_Element (GUI.CamioList, V, OK);
         Afegir_Element (GUI.CamioList, V, OK);
      elsif (Tipus = Ciclomotor) then
         Esborrar_Element (GUI.MotoList, V, OK);
         Afegir_Element (GUI.MotoList, V, OK);
      else
         Esborrar_Element (GUI.BikeList, V, OK);
         Afegir_Element (GUI.BikeList, V, OK);
      end if;
   end;
   
   procedure MoveCamioGrua(V: TVehicle; Tipus: Tipus_Vehicle) is
   begin
      GUI.CamioGrua := V;
   end;
   
   procedure MoveCamioBrossa(V: TVehicle; Tipus: Tipus_Vehicle) is
   begin
      GUI.CamioBrossa := V;
   end;
   
   procedure CamioGruaToGarage is
   begin
      GUI.CamioGrua.X := (GUI.CruillesDistance + GUI.StreetDistance*2)-GUI.GarageSize;
      GUI.CamioGrua.Y := GUI.StreetDistance-GUI.GarageSize;
   end;
   
   procedure CamioBrossaToGarage is
   begin
      GUI.CamioBrossa.X := (GUI.CruillesDistance + GUI.StreetDistance*2)-GUI.GarageSize;
      GUI.CamioBrossa.Y := GUI.StreetDistance-GUI.GarageSize;
   end;
   
   procedure CamioGruaTakeCar(V: TVehicle; Tipus: Tipus_Vehicle) is
      OK: Boolean;
   begin
      GUI.CamioGruaCarregat := true;
      if (Tipus = Cotxe) then
         Esborrar_Element (GUI.CarList, V, OK);
      elsif (Tipus = Camio) then
         Esborrar_Element (GUI.CamioList, V, OK);
      elsif (Tipus = Ciclomotor) then
         Esborrar_Element (GUI.MotoList, V, OK);
      else
         Esborrar_Element (GUI.BikeList, V, OK);
      end if;
   end;
   
   procedure CamioGruaLeaveCar is
   begin
      GUI.CamioGruaCarregat := false;
   end;
   
   procedure CamioGruaToStreet is
   begin
      GUI.CamioGrua.X := (GUI.CruillesDistance + GUI.StreetDistance*2)-GUI.GarageSize;
      GUI.CamioGrua.Y := GUI.StreetDistance;
   end;
   
   procedure CamioBrossaToStreet is
   begin
      GUI.CamioBrossa.X := (GUI.CruillesDistance + GUI.StreetDistance*2)-GUI.GarageSize;
      GUI.CamioBrossa.Y := GUI.StreetDistance;
   end;
   
   procedure Flow(Num: String) is
   begin
      Gdk.Threads.Enter;
      Set_Text(GUI.FlowLabel, utf8(Num & " %"));
      Gdk.Threads.Leave;
   end;
   
end InterficieVisual;

