
------------------------------------------------------------------------------------------
-- Practica de Enginyeria del Software II / Curs 2003 - 2004                            --
-- Simulador i Controlador de Transit en Temps Real                                     --
--                                                                                      --
-- Llista_Dinamica.adb: Implementacio del Tipus Abstracte de Dades llista, amb          --
--                      genericitat, doble encadenament, i punt d'interes que permet    --
--                      recorrer, obtenir i esborrar qualsevol element encuat.          --
-- Realitzat per: Francisco Jose Aguilar Celdran                                        --
--                Sergio Blanco Cuaresma                                                --
------------------------------------------------------------------------------------------


with Ada.Text_IO; use Ada.Text_IO;
with Unchecked_Deallocation;
with Ada.Exceptions; use Ada.Exceptions;

package body Llista_Dinamica is

   procedure Retornar_Memoria is new Unchecked_Deallocation (Object => node, Name => punter); 

   package I_IO is new Ada.Text_IO.Integer_IO(Integer); 
   use I_IO;

   ------------------
   -- Crear_Llista --
   ------------------

   procedure Crear_Llista (L: out Llista; OK: out Boolean) is
   begin
      L.Primer := NULL;
      L.Ultim := NULL;
      L.PDI := NULL;
      OK := True;
   exception
      when Storage_Error => OK := False;    -- No queda ms espai a la memoria 
   end Crear_Llista;

   --------------------
   -- Afegir_Element --
   --------------------

   procedure Afegir_Element (L: in out Llista; E: in Elem; OK: out Boolean) is
      Aux:punter;
   begin
      Aux := new Node;
      Copiar_Element (E,Aux.All.Info);
      if (L.Primer = NULL) then
         Aux.All.Esq := NULL;
         Aux.All.Drt := NULL;
         L.Primer := Aux;
            L.Ultim := Aux;
         L.PDI := Aux;         
      elsif (L.Primer.All.Drt = NULL) then
         Aux.All.Esq := L.Primer;
         Aux.All.Drt := NULL;
         L.Primer.All.Drt := Aux;
         L.Ultim := Aux;
         L.PDI := Aux;      
      else
         L.Ultim.All.Drt := Aux;     
         Aux.all.Esq := L.Ultim;
         Aux.All.Drt := NULL;  
         L.Ultim := Aux;
         OK := True;
      end if;
   exception
      when Storage_Error => OK := False;      -- No queda ms espai a la memoria
   end Afegir_Element;

   --------------------
   -- Primer_Element --
   --------------------

   function Primer_Element (L: in Llista) return Elem is
   begin
      return L.Primer.all.info;
   end Primer_Element;
   
   procedure Modificar_Primer_Element (L: in Llista; E: in Elem) is
   begin
      Copiar_Element (E,L.Primer.all.info);
   end Modificar_Primer_Element;

   ------------------
   -- Llista_Buida --
   ------------------

   function Llista_Buida (L: in Llista) return Boolean is
   begin
      return L.Primer = NULL;
   end Llista_Buida;

   ------------------------
   -- Establir_PDI_Inici --
   ------------------------

   procedure Establir_PDI_Inici (L: in out Llista) is
   begin
      L.PDI := L.Primer;
   end Establir_PDI_Inici;

   ---------------------
   -- Seguent_Element --
   ---------------------

   function Seguent_Element (L: in Llista) return Elem is
   begin
      if L.PDI /= L.Ultim then     
         return L.Pdi.All.Info;
      else
         return L.Ultim.All.Info;
      end if;
   end Seguent_Element;
   
   procedure Modificar_Seguent_Element (L: in Llista; E: in Elem) is
   begin
      Copiar_Element (E,L.Pdi.All.info);
   end Modificar_Seguent_Element;

   ---------------------------
   -- Augmentar_Posicio_PDI --
   ---------------------------

   procedure Augmentar_Posicio_PDI (L: in out Llista) is
   begin
      if (L.PDI /= L.Ultim and L.PDI /= NULL) then
         L.PDI := L.PDI.all.drt;
      else
         L.PDI := NULL;
      end if;
   end Augmentar_Posicio_PDI;

   ---------------------
   -- Queden_Elements --
   ---------------------

   function Queden_Elements (L: in Llista) return Boolean is
   begin
      return (L.PDI /= NULL);
   end Queden_Elements;
   
   ----------------------
   -- Element_Existent --
   ----------------------

   function Element_Existent (L: in Llista; E: in Elem) return Boolean is
      E2 : Elem;
      Llista_Aux : Llista;
      Trobat : Boolean := False;
   begin
      Llista_Aux := L;
      Establir_PDI_Inici (Llista_Aux);
      while (Queden_Elements (Llista_Aux) and Trobat = False) loop
         Copiar_Element (Seguent_Element(Llista_Aux),E2);
         if Comparar_Element (E,E2) then
            Trobat := True;   
         end if;
         Augmentar_Posicio_PDI (Llista_Aux);
      end loop;
      return Trobat;
   end Element_Existent;
   
   -------------------
   -- Ultim_Element --
   -------------------

   function Ultim_Element (L: in Llista) return Elem is
   begin
      return L.Ultim.all.info;
   end Ultim_Element;

   ----------------------
   -- Esborrar_Element --
   ----------------------

   procedure Esborrar_Element (L: in out Llista; E: in Elem; Ok: out Boolean) is
      E2 : Elem;
      Ant : Punter;
      Succ : Punter;
      Trobat : Boolean := False;
   begin
      if Llista_Buida (L) then
         raise Buida;            
      else
         Establir_PDI_Inici (L);
         while (Queden_Elements (L) and Trobat = False) loop
            Copiar_Element (Seguent_Element(L),E2);
            if Comparar_Element (E2,E) then
               Trobat := True;  
            else
               Augmentar_Posicio_PDI (L);
            end if;
         end loop;
         if (Trobat) then
            Ant := L.PDI.All.Esq;
            Succ := L.PDI.All.Drt;
            if (Ant /= NULL) then
               Ant.All.Drt := Succ;
            else
               L.Primer := Succ;
            end if;
            if (Succ /= NULL) then
               Succ.All.Esq := Ant;
            else
               L.Ultim := Ant;
            end if;
            Retornar_Memoria (L.PDI);
            OK := True;
         else
            raise No_Trobat;
         end if;  
     end if;
   exception
      when Buida => OK := False;
      when No_Trobat => OK := False;
      when E: others => Put_Line("Excepcio en Llista_Dinamica (Esborrar_Element) " & Exception_Name(E) & ", causada per: " & Exception_Message(E));
   end Esborrar_Element;

end Llista_Dinamica;

