
---------------------------------------------------------------------------------------------------
-- Practica de Enginyeria del Software II / Curs 2003 - 2004                                     --
-- Simulador i Controlador de Transit en Temps Real                                              --
--                                                                                               --
-- Llista_Dinamica.ads: Fitxer descriptor del modul Llista_Dinamica, amb les especificacions     --
--                      dels procediments, funcions i tipus de dades que s'empren en la          --
--                      implementacio d'aquest package.                                          --
-- Realitzat per: Francisco Jose Aguilar Celdran                                                 --
--                Sergio Blanco Cuaresma                                                         --
---------------------------------------------------------------------------------------------------


generic

   type Elem is limited private;
   with procedure Copiar_Element (E: in Elem; C: out Elem) is <>;
   with function Comparar_Element (E: in Elem; C: in Elem) return Boolean is <>;

package Llista_Dinamica is

   type Llista is limited private;

   procedure Crear_Llista (L: out Llista; OK: out Boolean); 
   procedure Afegir_Element (L: in out Llista; E: in Elem; OK: out Boolean);
   function Primer_Element (L: in Llista) return Elem;
   procedure Modificar_Primer_Element (L: in Llista; E: in Elem);
   function Llista_Buida (L: in Llista) return Boolean;
   procedure Establir_PDI_Inici (L: in out Llista);
   function Seguent_Element (L: in Llista) return Elem;
   procedure Modificar_Seguent_Element (L: in Llista; E: in Elem);
   procedure Augmentar_Posicio_PDI (L: in out Llista);
   function Queden_Elements (L: in Llista) return Boolean;
   function Element_Existent (L: in Llista; E: in Elem) return Boolean;
   function Ultim_Element (L: in Llista) return Elem;
   procedure Esborrar_Element (L: in out Llista; E: in Elem; Ok: out Boolean);

   Buida : exception;
   No_Trobat : exception;

private

   ----------------------------------------------------------------------------------
   -- Llista_Dinamica: Llista generica, doblement encadenada i amb punt d'interes. --
   --                   Permet esborrar un element en qualsevol posicio.           --
   ----------------------------------------------------------------------------------
   type Node;
   type Punter is access Node; 
   type Llista is record

      Primer : Punter;
      Ultim : Punter;
      PDI : Punter;   
   end record;


   type Node is record
      Info : Elem;
      Esq : Punter;
      Drt : Punter;
   end record;

end Llista_Dinamica;


