
------------------------------------------------------------------------------------------
-- Practica de Enginyeria del Software II / Curs 2003 - 2004                            --
-- Simulador i Controlador de Transit en Temps Real                                     --
--                                                                                      --
-- Tasques.adb: Fitxer que conte la logica de l'aplicacio, amb les diferents tasques    --
--              que composen el simulador, i diversos objectes protegits que hi estan   --
--              directament relacionats.                                                --
-- Realitzat per: Francisco Jose Aguilar Celdran                                        --
--                Sergio Blanco Cuaresma                                                --
------------------------------------------------------------------------------------------


with Ada.Text_IO; use Ada.Text_IO;
with Ada.Exceptions; use Ada.Exceptions;
with Ada.Real_Time; use Ada.Real_Time;
with Ada.Numerics.Float_Random; use Ada.Numerics.Float_Random;
with Funcions_Tasques; use Funcions_Tasques;
with Accions; use Accions;
with Ada.Integer_Text_IO; use Ada.Integer_Text_IO;
with System; use System;

package body Tasques is

   ----------------------
   -- Execucio_Tasques --
   ----------------------

   procedure Execucio_Tasques (C: in out Ciutat) is

      type Taula_Sincronitzacions is array(1..Num_Carrers(C,Horitzontal),1..Num_Carrers(C,Vertical)) of Sincronitzacions_Semafors;
      
      task type Tasca_Principal is
         entry Tasca_Preparada;
         entry Tasca_Finalitzada;
      end Tasca_Principal;
      type Acces_Tasca_Principal is access Tasca_Principal;
      Principal : Acces_Tasca_Principal;

      task type Tasca_Vehicle (Id: Positive; Tipus: Tipus_Vehicle; Principal: Acces_Tasca_Principal;
                               Espatllats: Vehicles_Espatllats; Vehicles_Creuant: Vehicles_Creuant_Cruilla) is
         entry Iniciar_Tasca;
      end Tasca_Vehicle;
      type Acces_Tasca_Vehicle is access Tasca_Vehicle;

      task type Tasca_Grua (Principal: Acces_Tasca_Principal; Espatllats: Vehicles_Espatllats;
                            Final_Simulador: Finalitzacio_Simulacio; Nombre_Vehicles: Natural;
                            Vehicles_Creuant: Vehicles_Creuant_Cruilla) is
	 entry Iniciar_Tasca;
         --entry Fi_Simulacio;       
      end Tasca_Grua;
      type Acces_Tasca_Grua is access Tasca_Grua;

      task type Tasca_Camio_Escombraries (Final_Simulador: Finalitzacio_Simulacio; Vehicles_Creuant: Vehicles_Creuant_Cruilla) is
         entry Iniciar_Tasca;
         --entry Fi_Simulacio;
      end Tasca_Camio_Escombraries;
      type Acces_Tasca_Escombraries is access Tasca_Camio_Escombraries;

      task type Tasca_Manegador_Cruilla (Fila, Columna: Positive; Estat_Inici_H: Estat_Semafor;
                                         Principal: Acces_Tasca_Principal;
                                         --Sincro_Cruilles: Taula_Sincronitzacions;
                                         Final_Simulador: Finalitzacio_Simulacio) is
         --entry Modificar_Inici_Periode (Color_Semafor: in Estat_Semafor; D: in Direccio);
         --entry Fi_Simulacio;
      end Tasca_Manegador_Cruilla;
      type Acces_Tasca_Cruilla is access Tasca_Manegador_Cruilla;
      
      task type Tasca_Controlador_Ciutat (Principal: Acces_Tasca_Principal;
                                          Espatllats: Vehicles_Espatllats;
                                          --Sincro_Cruilles: Taula_Sincronitzacions;
                                          Final_Simulador: Finalitzacio_Simulacio; Vehicles_Creuant: Vehicles_Creuant_Cruilla) is
         entry Iniciar_Tasca;
         --entry Vehicle_Espatllat(Id: in Positive; Tipus: Tipus_Vehicle; Pos_X, Pos_Y: in Positive);
         --entry Fi_Simulacio;
      end Tasca_Controlador_Ciutat;
      type Acces_Tasca_Controlador is access Tasca_Controlador_Ciutat;

      task type Actualitzador_Interficie (Final_Simulador: Finalitzacio_Simulacio) is
         entry Iniciar_Tasca;
      end Actualitzador_Interficie;
      type Acces_Actualitzador_Interficie is access Actualitzador_Interficie;
      
      type Conjunt_Cotxes is array (1..Nombre_Vehicles(C,Cotxe)) of Acces_Tasca_Vehicle;
      type Conjunt_Ciclomotors is array (1..Nombre_Vehicles(C,Ciclomotor)) of Acces_Tasca_Vehicle;
      type Conjunt_Camions is array (1..Nombre_Vehicles(C,Camio)) of Acces_Tasca_Vehicle;
      type Conjunt_Bicicletes is array (1..Nombre_Vehicles(C,Bicicleta)) of Acces_Tasca_Vehicle;
      type Conjunt_Semafors is array (1..Num_Cruilles_Ciutat(C)) of Acces_Tasca_Cruilla;

      Sincro_Cruilles : Taula_Sincronitzacions;
      Llavor : Generator;

      -------------------
      -- Tasca Vehicle -- 
      -------------------

      task body Tasca_Vehicle is
         Cruilla_Ini_H, Cruilla_Ini_V : Coordenada;
         Carrers_H, Carrers_V : Positive;
         Carrer_Ini_H, Carrer_Ini_V : Float;
         Carrer_H_Actual, Carrer_V_Actual : Positive;
         Velocitat : Positive;
         Distancia_Possible : Positive;
         Desviacio : Integer;
         Marge_Probabilitat : Float;
         Probabilitat : Float;
         Uts : Positive;
         Distancia_Actual : Natural;
         Temps_Actual : Time;
         Periode_Moviment : Time_Span;
         Temps_ArribadaSemafor : Time;
         Periode_EsperaSemafor : Time_Span;
         Periode_TotalEsperaSemafor : Duration;
         Pos_Actual : Coordenada;
         Direccio_Actual : Direccio;
         Metres_Entre_Cruilles : Positive;
         Metres_Desplasats : Positive;
         Cruilla_Actual : Coordenada;
         Creuament : Boolean;
         Desplasament, Metres_Fins_Semafor : Natural;
         Carrer_Actual : Positive;
         Girant_Carrer : Natural;
         Amplada_Carrer : Positive;
         Meitat_Tam_Vehicle : Positive;
	   Metres_Creuament_Cruilla : Natural;
	   Creuant_Cruilla : Boolean;
         Creuament_X, Creuament_Y : Positive;
      begin
         
         Posicionament_Aleatori_Inicial(C,Carrers_H,Carrers_V,Carrer_H_Actual,Carrer_V_Actual,
                                        Metres_Entre_Cruilles,Pos_Actual,Id,Tipus);
         Probabilitat := Random(Llavor);
         if Probabilitat <= 0.5 then
            Direccio_Actual := Horitzontal;
         else
            Direccio_Actual := Vertical;
         end if;

         -- Obtencio de parametres i espera inicialitzacio ciutat
         Velocitat := Velocitat_Maxima(C,Tipus);
         if Velocitat < 4 then
            Velocitat := 4;
	   end if;
         Marge_Probabilitat := Probabilitat_Seguir_Recte(C,Tipus);
         Distancia_Possible := Distancia_Mitja_Vehicle(C,Tipus);
         Desviacio := Desviacio_Distancia_Vehicle(C,Tipus);
         Uts := Unitat_Temps_Simulacio(C);
         Periode_Moviment := Milliseconds((1 / Uts) * 1000);
         Amplada_Carrer := Ample_Carrer(C);
         Distancia_Actual := 0;
         Girant_Carrer := 0;
         Distancia_Possible := Distancia_Possible + Desviacio;
         Metres_Desplasats := (Velocitat * 1000) / 3600;  -- Conversio km/h -> m/s
         Meitat_Tam_Vehicle := Meitat_Tamany_Vehicle(C,Tipus);
	   Metres_Creuament_Cruilla := 0;
	   Creuant_Cruilla := False;
         accept Iniciar_Tasca;
         
         Temps_Actual := Clock;
         while Distancia_Actual < Distancia_Possible loop
            if Girant_Carrer = 0 then
               
               case Direccio_Actual is
                  when Horitzontal => Carrer_Actual := Carrer_H_Actual;
                  when Vertical => Carrer_Actual := Carrer_V_Actual;
               end case;
               
               Vehicle_En_Cruilla(C,Pos_Actual.X,Pos_Actual.Y,Cruilla_Actual,Direccio_Actual,
                                  Carrer_Actual,Creuament,Metres_Fins_Semafor,Meitat_Tam_Vehicle);
                
               -- Si posicio actual esta en cruilla	
               if Creuament then 
                  
                  Temps_ArribadaSemafor := Clock;
                  Travessar_Semafor(C,Cruilla_Actual.X,Cruilla_Actual.Y,Direccio_Actual);
                  Periode_EsperaSemafor := Clock - Temps_ArribadaSemafor;
                  Periode_TotalEsperaSemafor := Periode_TotalEsperaSemafor + To_Duration(Periode_EsperaSemafor);
                  Actualitzar_Flow(C, Periode_TotalEsperaSemafor);
		            Actualitzar_Carrer(C,Carrer_H_Actual,Carrer_V_Actual,Direccio_Actual);

                  Vehicles_Creuant.Nou_Vehicle_Creuant(Cruilla_Actual.X,Cruilla_Actual.Y);
                  Metres_Creuament_Cruilla := 0;
                  Creuant_Cruilla := True;
                  Creuament_X := Cruilla_Actual.X;
                  Creuament_Y := Cruilla_Actual.Y;
                  
                  Probabilitat := Random(Llavor);
                  if (Probabilitat > Marge_Probabilitat) then
                     -- Es canviara la direccio en arribar al mig del carrer
                     Desplasament := Metres_Desplasats;
                     Girant_Carrer := Metres_Desplasats;
                  end if;
               else
                  if Metres_Fins_Semafor < Metres_Desplasats then
                     Desplasament := Metres_Fins_Semafor;
                  else
                     Desplasament := Metres_Desplasats;
                  end if;
               end if;
            -- Si s'ha de canviar de direccio, es mou en la direccio actual fins
            -- a la meitat de l'amplada del carrer (en la cruilla)
            else
               if (Girant_Carrer + Metres_Desplasats) > ((Amplada_Carrer / 2)  + Meitat_Tam_Vehicle) then -- Girant_Carrer + Metres_Desplasats > (Amplada_Carrer / 2) + Meitat_Tam_Vehicle 
                  Desplasament := ((Amplada_Carrer / 2) + Meitat_Tam_Vehicle) - Girant_Carrer;  -- (Amplada_Carrer / 2) + Meitat_Tam_Vehicle - Girant_Carrer;  !!! OJO : CAUSA D'EXCEPCIO EN ALGUNS CASOS, PENSAR-HO !!!!
                  Girant_Carrer := Girant_Carrer + Desplasament;
               else
                  Girant_Carrer := Girant_Carrer + Metres_Desplasats;
                  Desplasament := Metres_Desplasats;
               end if;
            end if;
            Distancia_Actual := Distancia_Actual + Desplasament;
            Pos_Actual := Actualitzar_Coordenades(C,Direccio_Actual,Pos_Actual,Desplasament,
                                                  Carrer_H_Actual,Carrer_V_Actual);
            Moure_Vehicle(C,Pos_Actual,Id,Tipus,False,False);
            if Creuant_Cruilla then
               Metres_Creuament_Cruilla := Metres_Creuament_Cruilla + Desplasament;
               if Metres_Creuament_Cruilla >= Amplada_Carrer then
                  Creuant_Cruilla := False;
                  Metres_Creuament_Cruilla := 0;
                  Vehicles_Creuant.Vehicle_Surt_De_Cruilla(Creuament_X,Creuament_Y);
               end if;
            end if;

            -- Canviem la direccio actual en arribar a la meitat del carrer
            -- en haver decidit modificar-la quan s'ha travessat la cruilla   
            if Girant_Carrer = (Amplada_Carrer / 2) + Meitat_Tam_Vehicle then
               case Direccio_Actual is
                  when Horitzontal => Direccio_Actual := Vertical;
                  when Vertical => Direccio_Actual := Horitzontal;
               end case;
               Girant_Carrer := 0;
            end if;
            delay until (Temps_Actual + Periode_Moviment);
            Temps_Actual := Clock;
         end loop;
         Vehicle_Avariat(C,Pos_Actual,Id,Tipus);   -- Acci per interficie
         if Direccio_Actual = Horitzontal then
            Espatllats.Nou_Vehicle_Espatllat(Id,Tipus,Pos_Actual.X,Pos_Actual.Y,
                                             Carrer_H_Actual,Horitzontal);
         else
            Espatllats.Nou_Vehicle_Espatllat(Id,Tipus,Pos_Actual.X,Pos_Actual.Y,
                                             Carrer_V_Actual,Vertical);
         end if;
         Principal.Tasca_Finalitzada;

      exception
         when E: others => Put_Line("Excepcio en Tasca_Vehicle " & Exception_Name(E) & ", causada per: " & Exception_Message(E));
                           Principal.Tasca_Finalitzada;
      end Tasca_Vehicle;

      ----------------
      -- Tasca Grua --
      ----------------

      task body Tasca_Grua is

         Fi : Boolean := False;
         OK : Boolean := False;
         Dades_Avaria : Info_Avaria;
         Velocitat : Positive;
         Uts : Positive;
         Pos_Garatge, Pos_Taller : Coordenada;
         Carrer_Garatge, Carrer_Taller : Positive;
         Dir_Garatge, Dir_Taller : Direccio;
         Pos_Actual : Coordenada;
         Cruilla_Actual : Coordenada;
         Metres_Desplasats : Positive;
         Temps_Inicial, Temps_Actual : Time;
         Direccio_Actual : Direccio;
         Creuament : Boolean;
         Carrer_H_Actual, Carrer_V_Actual : Positive;
         Periode_Moviment : Time_Span;
         Temps_ArribadaSemafor : Time;
         Periode_EsperaSemafor : Time_Span;
         Periode_TotalEsperaSemafor : Duration;
         Coord_Avaria : Coordenada;
         Direccio_Inicial : Direccio;
         Desplasament, Metres_Fins_Semafor : Natural;
         Carrer_Actual : Positive;
         Girant_Carrer : Natural;
         Amplada_Carrer : Positive;
         Meitat_Tam_Vehicle : Positive;
         Vehicles_recollits: Natural;
	   Metres_Creuament_Cruilla : Natural;
	   Creuant_Cruilla : Boolean;
         Creuament_X, Creuament_Y : Positive;
         
         procedure Executar_Recorregut (Pos_Desti: in Coordenada; Carrer_Desti: in Positive; Dir_Desti: in Direccio) is
            Dif_X_Abans, Dif_Y_Abans : Integer;
            Dif_X_Despres, Dif_Y_Despres : Integer;
         begin
            while (Pos_Actual.X /= Pos_Desti.X or Pos_Actual.Y /= Pos_Desti.Y) loop
               if Girant_Carrer = 0 then
                  case Direccio_Actual is
                     when Horitzontal => Carrer_Actual := Carrer_H_Actual;
                     when Vertical => Carrer_Actual := Carrer_V_Actual;
                  end case;
                  Vehicle_En_Cruilla(C,Pos_Actual.X,Pos_Actual.Y,Cruilla_Actual,Direccio_Actual,
                                     Carrer_Actual,Creuament,Metres_Fins_Semafor,Meitat_Tam_Vehicle);
                  if Creuament then
                     
                     Temps_ArribadaSemafor := Clock;
                     Travessar_Semafor(C,Cruilla_Actual.X,Cruilla_Actual.Y,Direccio_Actual);
                     Periode_EsperaSemafor := Clock - Temps_ArribadaSemafor;
                     Periode_TotalEsperaSemafor := Periode_TotalEsperaSemafor + To_Duration(Periode_EsperaSemafor);
                     Actualitzar_Flow(C, Periode_TotalEsperaSemafor);

                     Vehicles_Creuant.Nou_Vehicle_Creuant(Cruilla_Actual.X,Cruilla_Actual.Y);
                     Metres_Creuament_Cruilla := 0;
                     Creuant_Cruilla := True;
                     Creuament_X := Cruilla_Actual.X;
                     Creuament_Y := Cruilla_Actual.Y;

                     Actualitzar_Carrer(C,Carrer_H_Actual,Carrer_V_Actual,Direccio_Actual);
                     Direccio_Inicial := Direccio_Actual;
                     Decidir_Direccio_Grua(C,Pos_Actual,Dades_Avaria.Pos_X,Dades_Avaria.Pos_Y,
                                           Carrer_H_Actual,Carrer_V_Actual,Direccio_Actual,
                                           Carrer_Desti,Dir_Desti);
                     if (Direccio_Inicial /= Direccio_Actual) then
                        -- Es canviara la direccio en arribar al mig del carrer
                        Desplasament := Metres_Desplasats;
                        Girant_Carrer := Metres_Desplasats;
                        Direccio_Actual := Direccio_Inicial; 
                     end if;
                  else
                     if Metres_Fins_Semafor < Metres_Desplasats then
                        Desplasament := Metres_Fins_Semafor;
                     else
                        Desplasament := Metres_Desplasats;
                     end if;
                  end if;
               -- Si s'ha de canviar de direccio, es mou en la direccio actual fins
               -- a la meitat de l'amplada del carrer (en la cruilla)   
               else
                  if (Girant_Carrer + Metres_Desplasats) > ((Amplada_Carrer / 2) + Meitat_Tam_Vehicle) then
                     Desplasament := ((Amplada_Carrer / 2) + Meitat_Tam_Vehicle) - Girant_Carrer;
                     Girant_Carrer := Girant_Carrer + Desplasament;
                  else
                     Girant_Carrer := Girant_Carrer + Metres_Desplasats;
                     Desplasament := Metres_Desplasats;
                  end if;
               end if;
               Dif_Y_Abans := Pos_Actual.Y - Pos_Desti.Y;
               Dif_X_Abans := Pos_Actual.X - Pos_Desti.X; 
               Pos_Actual := Actualitzar_Coordenades(C,Direccio_Actual,Pos_Actual,Desplasament,
                                                     Carrer_H_Actual,Carrer_V_Actual);

               -- Centrar-se exactament sobre el vehicle en aproximar-s'hi
               Dif_Y_Despres := Pos_Actual.Y - Pos_Desti.Y;
               Dif_X_Despres := Pos_Actual.X - Pos_Desti.X;
               if Direccio_Actual = Horitzontal and Dir_Desti = Horitzontal and Carrer_H_Actual = Carrer_Desti then
                  if ((Dif_X_Abans > 0 and Dif_X_Despres < 0) or
                     (Dif_X_Abans < 0 and Dif_X_Despres > 0)) then
                     if abs(Dif_X_Abans - Dif_X_Despres) <= Metres_Desplasats then
                        Pos_Actual.X := Pos_Desti.X;
                     end if;
                  end if;
               elsif Direccio_Actual = Vertical and Dir_Desti = Vertical and Carrer_V_Actual = Carrer_Desti then
                  if ((Dif_Y_Abans > 0 and Dif_Y_Despres < 0) or
                     (Dif_Y_Abans < 0 and Dif_Y_Despres > 0)) then
                     if abs(Dif_Y_Abans - Dif_Y_Despres) <= Metres_Desplasats then
                        Pos_Actual.Y := Pos_Desti.Y;
                     end if;
                  end if;
               end if;

               delay until (Temps_Actual + Periode_Moviment);
               Temps_Actual := Clock;
               Moure_Vehicle(C,Pos_Actual,1,Camio,True,False);
               
               if Creuant_Cruilla then
                  Metres_Creuament_Cruilla := Metres_Creuament_Cruilla + Desplasament;
                  if Metres_Creuament_Cruilla >= Amplada_Carrer then
                     Creuant_Cruilla := False;
                     Metres_Creuament_Cruilla := 0;
                     Vehicles_Creuant.Vehicle_Surt_De_Cruilla(Creuament_X,Creuament_Y);
                  end if;
               end if;
               
               -- Canviem la direccio actual en arribar a la meitat del carrer
               -- en haver decidit modificar-la quan s'ha travessat la cruilla   
               if Girant_Carrer >= ((Amplada_Carrer / 2) + Meitat_Tam_Vehicle) then
                  case Direccio_Actual is
                     when Horitzontal => Direccio_Actual := Vertical;
                     when Vertical => Direccio_Actual := Horitzontal;
                  end case;
                  Girant_Carrer := 0;
               end if;
            end loop;
         end Executar_Recorregut;

      begin
         Velocitat := Velocitat_Maxima(C,Camio) / 2;
         if Velocitat < 4 then
            Velocitat := 4;
	      end if;
         Uts := Unitat_Temps_Simulacio(C);
         Periode_Moviment := Milliseconds((1 / UTS) * 1000);
         Metres_Desplasats := (Velocitat * 1000) / 3600;  -- Conversio km/h -> m/s
         Pos_Garatge := Coordenades_Garatge(C);
         Pos_Taller := Coordenades_Taller(C);
         Amplada_Carrer := Ample_Carrer(C);
         Meitat_Tam_Vehicle := Meitat_Tamany_Vehicle(C,Camio);
	   Metres_Creuament_Cruilla := 0;
	   Creuant_Cruilla := False;
         Girant_Carrer := 0;

         accept Iniciar_Tasca;
         Temps_Actual := Clock;
         Vehicles_recollits := 0;
         Obtenir_Carrer_Garatge(C,Carrer_Garatge,Dir_Garatge);
         Obtenir_Carrer_Taller(C,Carrer_Taller,Dir_Taller);
         Pos_Taller.X := Pos_Taller.X + Meitat_Tam_Vehicle;
         while Nombre_Vehicles /= Vehicles_Recollits loop
            Espatllats.Activar_Grua(Dades_Avaria);
            Pos_Actual := Pos_Garatge;
	         Direccio_Actual := Dir_Garatge;
               
            -- Sortir del garatge
            Treure_Vehicle_Garatge(C,True,False);
            Carrers_H_V_Immoble(C,Carrer_Garatge,Dir_Garatge,Pos_Garatge,
                                Carrer_H_Actual,Carrer_V_Actual);

            -- Primer Recorregut fins al vehicle avariat
            Coord_Avaria.X := Dades_Avaria.Pos_X;
            Coord_Avaria.Y := Dades_Avaria.Pos_Y;

            Executar_Recorregut(Coord_Avaria,Dades_Avaria.Num_Carrer,Dades_Avaria.Orientacio);


            -- La Grua carrega el vehicle avariat
            Recollir_Vehicle_Avariat(C,Pos_Actual,Dades_Avaria.Id_Vehicle,Dades_Avaria.Tipus);
            delay until (Temps_Actual + Milliseconds((60 / Uts) * 1000));
            Temps_Actual := Clock;

            -- Segon Recorregut fins al taller
            Executar_Recorregut(Pos_Taller,Carrer_Taller,Dir_Taller);               

            -- La Grua descarrega el vehicle avariat en el taller
            delay until (Temps_Actual + Milliseconds((60 / Uts) * 1000));
            Temps_Actual := Clock;
            Deixar_Avariat_En_Taller(C,Dades_Avaria.Id_Vehicle,Dades_Avaria.Tipus);
            delay until (Temps_Actual + Milliseconds((30 / Uts) * 1000));
            Temps_Actual := Clock;

            -- Tercer Recorregut fins al garatge
            Executar_Recorregut(Pos_Garatge,Carrer_Garatge,Dir_Garatge);

            -- Entrar al garatge 
            delay until (Temps_Actual + Milliseconds((30 / Uts) * 1000));
            Temps_Actual := Clock;
            Guardar_Vehicle_Garatge(C,True,False);
            delay until (Temps_Actual + Milliseconds((60 / Uts) * 1000));
            Temps_Actual := Clock;

            Vehicles_Recollits := Vehicles_Recollits + 1;
         end loop;
         
         Principal.Tasca_Finalitzada;

      exception
         when E: others => Put_Line("Excepcio en Tasca_Grua " & Exception_Name(E) & ", causada per: " & Exception_Message(E));
                           Final_Simulador.Indicar_Fi_Simulacio;
      end Tasca_Grua;
   
      ------------------------------
      -- Tasca_Camio_Escombraries --
      ------------------------------

      task body Tasca_Camio_Escombraries is
         Fi : Boolean := False;
         Hores_Periode : Positive;
         Minuts_Contenidor : Natural;
         Temps_Actual : Time;
         Uts : Positive;
         Periode_Escombraries : Time_Span;
         Periode_Inicial : Time_Span;
         Periode_Moviment : Time_Span;
         Temps_Contenidor : Time_Span;
         Velocitat : Positive;
         Metres_Entre_Cruilles : Positive;
         Pos_Garatge, Pos_Taller : Coordenada;
         Metres_Desplasats : Positive;
         Carrer_H_Actual, Carrer_V_Actual : Positive;
         Cruilles_Actuals : Natural;
         Carrer_Garatge, Carrer_Taller : Positive;
         Dir_Garatge, Dir_Taller : Direccio;
         Pos_Actual : Coordenada;
         Cruilla_Actual : Coordenada;
         Direccio_Actual : Direccio;
         Creuament : Boolean;
         Comptador_Contenidor : Integer;
         Direccio_Inicial : Direccio;
         Desplasament, Metres_Fins_Semafor : Natural;
         Carrer_Actual : Positive;
         Girant_Carrer : Natural;
         Amplada_Carrer : Positive;
         Meitat_Tam_Vehicle : Positive;
         Tornar_A_Garatge : Boolean;
         Inici : Boolean;
	      Metres_Creuament_Cruilla : Natural;
	      Creuant_Cruilla : Boolean;
         Creuament_X, Creuament_Y : Positive;
         Temps_ArribadaSemafor : Time;
         Periode_EsperaSemafor : Time_Span;
         Periode_TotalEsperaSemafor : Duration;
         
         function Actualitzar_Distancia_Contenidor(Dir_Contraria: in Direccio; Mode: in Natural) return Natural is
            Carrer_Contrari : Natural;
            Metres_Fins_Contenidor : Natural;
            Sentit_H : Orientacio_Horitzontal;
            Sentit_V : Orientacio_Vertical;
         begin
            if Direccio_Actual = Horitzontal then
               Carrer_Contrari := Carrer_V_Actual;
               Sentit_H := Orientacio_Carrer_H(C,Carrer_H_Actual);
            else
               Carrer_Contrari := Carrer_H_Actual;
               Sentit_V := Orientacio_Carrer_V(C,Carrer_V_Actual);
            end if;
            
            if ((Carrer_Contrari = Num_Carrers(C,Dir_Contraria)) and ((Sentit_V = Abaix and Direccio_Actual = Vertical) or (Sentit_H = Dreta and Direccio_Actual = Horitzontal))) or
               ((Carrer_Contrari = 1) and ((Sentit_V = Amunt and Direccio_Actual = Vertical) or (Sentit_H = Esquerra and Direccio_Actual = Horitzontal))) then
               Metres_Fins_Contenidor := Metres_Entre_Cruilles;
               if Mode = 1 then
                  Comptador_Contenidor := 0;
               else
                  Comptador_Contenidor := - Meitat_Tam_Vehicle;
               end if;
            else
               Metres_Fins_Contenidor := ((Amplada_Carrer / 2) + (Metres_Entre_Cruilles / 2)) * 2; --(Amplada_Carrer / 2) + Metres_Entre_Cruilles + Meitat_Tam_Vehicle;
            end if;
            return Metres_Fins_Contenidor;
         end Actualitzar_Distancia_Contenidor;
   
         procedure Avansar_Fins_Semafor is
         begin
            Creuament := False;
            while not Creuament loop
               Vehicle_En_Cruilla(C,Pos_Actual.X,Pos_Actual.Y,Cruilla_Actual,Direccio_Actual,
                                  Carrer_H_Actual,Creuament,Metres_Fins_Semafor,Meitat_Tam_Vehicle);
               if Creuament then
                  
                  Temps_ArribadaSemafor := Clock;
                  Travessar_Semafor(C,Cruilla_Actual.X,Cruilla_Actual.Y,Direccio_Actual);
                  Periode_EsperaSemafor := Clock - Temps_ArribadaSemafor;
                  Periode_TotalEsperaSemafor := Periode_TotalEsperaSemafor + To_Duration(Periode_EsperaSemafor);
                  Actualitzar_Flow(C, Periode_TotalEsperaSemafor);
                     
                  Vehicles_Creuant.Nou_Vehicle_Creuant(Cruilla_Actual.X,Cruilla_Actual.Y);
                  Metres_Creuament_Cruilla := 0;
                  Creuant_Cruilla := True;
                  Creuament_X := Cruilla_Actual.X;
                  Creuament_Y := Cruilla_Actual.Y;
                  Actualitzar_Carrer(C,Carrer_H_Actual,Carrer_V_Actual,Direccio_Actual);
               else
                  if Metres_Fins_Semafor < Metres_Desplasats then
                     Desplasament := Metres_Fins_Semafor;
                  else
                     Desplasament := Metres_Desplasats;
                  end if;
               end if;
               Pos_Actual := Actualitzar_Coordenades(C,Direccio_Actual,Pos_Actual,Desplasament,
                                                     Carrer_H_Actual,Carrer_V_Actual);
               delay until (Temps_Actual + Periode_Moviment);
               Temps_Actual := Clock;
               Moure_Vehicle(C,Pos_Actual,1,Camio,False,True);
               if Creuant_Cruilla then
                  Metres_Creuament_Cruilla := Metres_Creuament_Cruilla + Desplasament;
                  if Metres_Creuament_Cruilla >= Amplada_Carrer then
                     Creuant_Cruilla := False;
                     Metres_Creuament_Cruilla := 0;
                     Vehicles_Creuant.Vehicle_Surt_De_Cruilla(Creuament_X,Creuament_Y);
                  end if;
               end if;
            end loop;
         end Avansar_Fins_Semafor;

         procedure Moure_Fins_Meitat_Cruilla is
            Metres_Actuals : Natural := 0;
         begin
            while Metres_Actuals < ((Amplada_Carrer / 2) + Meitat_Tam_Vehicle) loop
               if (Metres_Actuals + Metres_Desplasats) > ((Amplada_Carrer / 2) + Meitat_Tam_Vehicle) then
                  Desplasament := ((Amplada_Carrer / 2) + Meitat_Tam_Vehicle) - Metres_Actuals;
                  Metres_Actuals := Metres_Actuals + Desplasament;
               else
                  Metres_Actuals := Metres_Actuals + Metres_Desplasats;
                  Desplasament := Metres_Desplasats;
               end if;
               Pos_Actual := Actualitzar_Coordenades(C,Direccio_Actual,Pos_Actual,Desplasament,
                                                     Carrer_H_Actual,Carrer_V_Actual);
               delay until (Temps_Actual + Periode_Moviment);
               Temps_Actual := Clock;
               Moure_Vehicle(C,Pos_Actual,1,Camio,False,True);
               if Creuant_Cruilla then
                  Metres_Creuament_Cruilla := Metres_Creuament_Cruilla + Desplasament;
                  if Metres_Creuament_Cruilla >= Amplada_Carrer then
                     Creuant_Cruilla := False;
                     Metres_Creuament_Cruilla := 0;
                     Vehicles_Creuant.Vehicle_Surt_De_Cruilla(Creuament_X,Creuament_Y);
                  end if;
               end if;
            end loop;
         end Moure_Fins_Meitat_Cruilla;

         procedure Moure_Fins_Seguent_Carrer (Seguent_Dir: in Direccio; Parada: in Boolean) is
            Metres_Fins_Contenidor : Natural;
            Dir_Contraria : Direccio;
            Carrer_Contrari : Natural;
            Parades : Natural;
            Sentit_H : Orientacio_Horitzontal;
            Sentit_V : Orientacio_Vertical;
         begin
            if Direccio_Actual = Horitzontal then
               Dir_Contraria := Vertical;
               Carrer_Contrari := Carrer_V_Actual;
               Sentit_H := Orientacio_Carrer_H(C,Carrer_H_Actual);
            else
               Carrer_Contrari := Carrer_H_Actual;
               Dir_Contraria := Horitzontal;
               Sentit_V := Orientacio_Carrer_V(C,Carrer_V_Actual);
            end if;
            if Parada then
               if ((Carrer_Contrari = Num_Carrers(C,Dir_Contraria)) and ((Sentit_V = Abaix and Direccio_Actual = Vertical) or (Sentit_H = Dreta and Direccio_Actual = Horitzontal))) or
                  ((Carrer_Contrari = 1) and ((Sentit_V = Amunt and Direccio_Actual = Vertical) or (Sentit_H = Esquerra and Direccio_Actual = Horitzontal))) then
                  Parades := 2;
               else
                  Parades := 1;
               end if;
            else
               Parades := 0;
            end if;  
            Moure_Fins_Meitat_Cruilla;
            Direccio_Actual := Seguent_Dir;
            Creuament := False;
            Comptador_Contenidor := 0;
            Metres_Fins_Contenidor := (Amplada_Carrer / 2) + (Metres_Entre_Cruilles / 2);
            while not Creuament loop
               case Direccio_Actual is
                  when Horitzontal => Carrer_Actual := Carrer_H_Actual;
                  when Vertical => Carrer_Actual := Carrer_V_Actual;
               end case;
               Vehicle_En_Cruilla(C,Pos_Actual.X,Pos_Actual.Y,Cruilla_Actual,Direccio_Actual,
                                  Carrer_Actual,Creuament,Metres_Fins_Semafor,Meitat_Tam_Vehicle);
               if Creuament then
               
                  Temps_ArribadaSemafor := Clock;
                  Travessar_Semafor(C,Cruilla_Actual.X,Cruilla_Actual.Y,Direccio_Actual);
                  Periode_EsperaSemafor := Clock - Temps_ArribadaSemafor;
                  Periode_TotalEsperaSemafor := Periode_TotalEsperaSemafor + To_Duration(Periode_EsperaSemafor);
                  Actualitzar_Flow(C, Periode_TotalEsperaSemafor);
                  
                  Vehicles_Creuant.Nou_Vehicle_Creuant(Cruilla_Actual.X,Cruilla_Actual.Y);
                  Metres_Creuament_Cruilla := 0;
                  Creuant_Cruilla := True;
                  Creuament_X := Cruilla_Actual.X;
                  Creuament_Y := Cruilla_Actual.Y;                  
                  Actualitzar_Carrer(C,Carrer_H_Actual,Carrer_V_Actual,Direccio_Actual);
                  Desplasament := Metres_Desplasats;
               else
                  if Parada = True then
                     if Metres_Fins_Semafor < Metres_Desplasats then
                        Desplasament := Metres_Fins_Semafor;
                        Comptador_Contenidor := Comptador_Contenidor + Metres_Fins_Semafor;
                     elsif Metres_Fins_Contenidor < Metres_Desplasats then
                        Desplasament := Metres_Desplasats - Metres_Fins_Contenidor;
                        Comptador_Contenidor := Comptador_Contenidor + (Metres_Desplasats - Metres_Fins_Contenidor);
                     else
                        Desplasament := Metres_Desplasats;
                        Comptador_Contenidor := Comptador_Contenidor + Metres_Desplasats;
                     end if;                  
                  else
                     if Metres_Fins_Semafor < Metres_Desplasats then
                        Desplasament := Metres_Fins_Semafor;
                     else
                        Desplasament := Metres_Desplasats;
                     end if;
                  end if;
               end if;
               Pos_Actual := Actualitzar_Coordenades(C,Direccio_Actual,Pos_Actual,Desplasament,
                                                     Carrer_H_Actual,Carrer_V_Actual);
               delay until (Temps_Actual + Periode_Moviment);
               Temps_Actual := Clock;
               Moure_Vehicle(C,Pos_Actual,1,Camio,False,True);
               if Creuant_Cruilla then
                  Metres_Creuament_Cruilla := Metres_Creuament_Cruilla + Desplasament;
                  if Metres_Creuament_Cruilla >= Amplada_Carrer then
                     Creuant_Cruilla := False;
                     Metres_Creuament_Cruilla := 0;
                     Vehicles_Creuant.Vehicle_Surt_De_Cruilla(Creuament_X,Creuament_Y);
                  end if;
               end if;
               if Parada = True then
                  if (Comptador_Contenidor >= Metres_Fins_Contenidor) and Parades > 0 then 
                     delay Duration((Minuts_Contenidor * 60) / Uts);
                     Parades := Parades - 1;
                     if Parades > 0 then
                        Comptador_Contenidor := 0;
                        Metres_Fins_Contenidor := Metres_Entre_Cruilles;
                     end if;
                  end if;               
               end if;
            end loop;
            if not Tornar_A_Garatge then
               Moure_Fins_Meitat_Cruilla;
            end if;
            if Direccio_Actual = Horitzontal then
               Direccio_Actual := Vertical;
            else
               Direccio_Actual := Horitzontal;
            end if;
         end Moure_Fins_Seguent_Carrer;

         procedure Recorregut_Escombraries (Dir: in Direccio; Parar_En_Sentit_Contrari: in Boolean) is
            Dir_Contraria : Direccio;
            Carrers_Recollits : Natural;
            Metres_Fins_Contenidor : Natural;
         begin
            if Dir = Horitzontal then
               Dir_Contraria := Vertical;
               Carrers_Recollits := 0;
            else
               Dir_Contraria := Horitzontal;
               Carrers_Recollits := 1;
            end if;
            -- Recorregut de recollida d'escombraries en la direccio demanada
            while Carrers_Recollits < Num_Carrers(C,Dir) loop
               Cruilles_Actuals := 0;
               Metres_Fins_Contenidor := (Amplada_Carrer / 2) + (Metres_Entre_Cruilles / 2);
               Comptador_Contenidor := 0;
               Direccio_Actual := Dir;
               -- Recorrer el Carrer Horitzontal, parant en cada contenidor
               while Cruilles_Actuals < Num_Carrers(C,Dir_Contraria) loop
                  case Direccio_Actual is
                     when Horitzontal => Carrer_Actual := Carrer_H_Actual;
                     when Vertical => Carrer_Actual := Carrer_V_Actual;
                  end case;
                  Vehicle_En_Cruilla(C,Pos_Actual.X,Pos_Actual.Y,Cruilla_Actual,Direccio_Actual,
                                     Carrer_Actual,Creuament,Metres_Fins_Semafor,Meitat_Tam_Vehicle);
                  if Creuament then
                  
                     Temps_ArribadaSemafor := Clock;
                     Travessar_Semafor(C,Cruilla_Actual.X,Cruilla_Actual.Y,Direccio_Actual);
                     Periode_EsperaSemafor := Clock - Temps_ArribadaSemafor;
                     Periode_TotalEsperaSemafor := Periode_TotalEsperaSemafor + To_Duration(Periode_EsperaSemafor);
                     Actualitzar_Flow(C, Periode_TotalEsperaSemafor);
                     
                     Vehicles_Creuant.Nou_Vehicle_Creuant(Cruilla_Actual.X,Cruilla_Actual.Y);
                     Metres_Creuament_Cruilla := 0;
                     Creuant_Cruilla := True;
                     Creuament_X := Cruilla_Actual.X;
                     Creuament_Y := Cruilla_Actual.Y;                     
                     Actualitzar_Carrer(C,Carrer_H_Actual,Carrer_V_Actual,Direccio_Actual);
                     Cruilles_Actuals := Cruilles_Actuals + 1;
                     Metres_Fins_Contenidor := Actualitzar_Distancia_Contenidor(Dir_Contraria,2);
                     Desplasament := Metres_Desplasats;
                     Comptador_Contenidor := Comptador_Contenidor+ Metres_Desplasats;
                  else
                     if Metres_Fins_Semafor < Metres_Desplasats then
                        Desplasament := Metres_Fins_Semafor;
                        Comptador_Contenidor := Comptador_Contenidor + Metres_Fins_Semafor;
                     elsif Metres_Fins_Contenidor < Metres_Desplasats then
                        Desplasament := Metres_Desplasats - Metres_Fins_Contenidor;
                        Comptador_Contenidor := Comptador_Contenidor + (Metres_Desplasats - Metres_Fins_Contenidor);
                     else
                        Desplasament := Metres_Desplasats;
                        Comptador_Contenidor := Comptador_Contenidor + Metres_Desplasats;
                     end if;
                  end if;
                  if Cruilles_Actuals < Num_Carrers(C,Dir_Contraria) then
                     Pos_Actual := Actualitzar_Coordenades(C,Direccio_Actual,Pos_Actual,
                                                           Desplasament,Carrer_H_Actual,
                                                           Carrer_V_Actual);
                     delay until (Temps_Actual + Periode_Moviment);
                     Temps_Actual := Clock;
                     Moure_Vehicle(C,Pos_Actual,1,Camio,False,True);
                     if Creuant_Cruilla then
                        Metres_Creuament_Cruilla := Metres_Creuament_Cruilla + Desplasament;
                        if Metres_Creuament_Cruilla >= Amplada_Carrer then
                           Creuant_Cruilla := False;
                           Metres_Creuament_Cruilla := 0;
                           Vehicles_Creuant.Vehicle_Surt_De_Cruilla(Creuament_X,Creuament_Y);
                        end if;
                     end if;                     
                     if Comptador_Contenidor >= Metres_Fins_Contenidor then
                        Comptador_Contenidor := 0;
                        Metres_Fins_Contenidor := Actualitzar_Distancia_Contenidor(Dir_Contraria,1); 
                        delay Duration((Minuts_Contenidor * 60) / Uts);
                     end if;
                  end if;
               end loop;
               -- Moure Escombriaire Seguent Carrer en la direccio establerta
               if (Carrers_Recollits = Num_Carrers(C,Dir) - 1) and (Dir = Vertical) then
                  Tornar_A_Garatge := True;
               end if;
               Moure_Fins_Seguent_Carrer(Dir_Contraria,Parar_En_Sentit_Contrari);
               Carrers_Recollits := Carrers_Recollits + 1;
            end loop;            
         end Recorregut_Escombraries;

      begin
         Hores_Periode := Periode_Camio_Escombraries(C);
         Minuts_Contenidor := Temps_En_Contenidor(C);
         Uts := Unitat_Temps_Simulacio(C);
         Metres_Entre_Cruilles := Separacio_Entre_Cruilles(C);
         Pos_Garatge := Coordenades_Garatge(C);
         Pos_Taller := Coordenades_Taller(C);
         Periode_Moviment := Milliseconds((1 / UTS) * 1000);
         Periode_Escombraries := Milliseconds(((Hores_Periode * 3600) / Uts) * 1000);
         Periode_Inicial := Milliseconds(((2 * 60) / Uts) * 1000);
         Temps_Contenidor := Milliseconds(((Minuts_Contenidor * 60) / Uts) * 1000);
         Velocitat := Velocitat_Maxima(C,Camio) / 2;
	   if Velocitat < 4 then
            Velocitat := 4;
	   end if;
         Metres_Desplasats := (Velocitat * 1000) / 3600;            -- Conversio km/h -> m/s
         Amplada_Carrer := Ample_Carrer(C);
         Meitat_Tam_Vehicle := Meitat_Tamany_Vehicle(C,Camio);
         Metres_Creuament_Cruilla := 0;
	      Creuant_Cruilla := False;
         Girant_Carrer := 0;
         Inici := True;
         accept Iniciar_Tasca;
         Temps_Actual := Clock;
         while not Fi loop
            select
               Final_Simulador.Comprovar_Fi_Anticipada;
               Fi := True;
            then abort
               if Inici then
                  Inici := False;
                  delay until Temps_Actual + Periode_Inicial;
               else
                  delay until Temps_Actual + Periode_Escombraries;
               end if;
            end select;
            if not Fi then
               Temps_Actual := Clock; 
               Treure_Vehicle_Garatge(C,False,True);        -- Sortir del Garatge
               Obtenir_Carrer_Garatge(C,Carrer_Garatge,Dir_Garatge);
               Carrers_H_V_Immoble(C,Carrer_Garatge,Dir_Garatge,Pos_Garatge,
                                   Carrer_H_Actual,Carrer_V_Actual);
               Pos_Actual := Pos_Garatge;        
               Direccio_Actual := Dir_Garatge;
               Tornar_A_Garatge := False;

               Avansar_Fins_Semafor;
               Moure_Fins_Meitat_Cruilla;
               Recorregut_Escombraries(Horitzontal,True);   -- Recollir escombraries carrers horitzontals
               Direccio_Actual := Horitzontal;
               Avansar_Fins_Semafor;
               Moure_Fins_Meitat_Cruilla;
               Recorregut_Escombraries(Vertical,False);     -- Recollir escombraries carrers verticals
               Guardar_Vehicle_Garatge(C,False,True);
               Fi := Final_Simulador.Fi_Anticipada;
            end if;
         end loop;
         Principal.Tasca_Finalitzada;
      exception
         when E: others => Put_Line("Excepcio en Tasca_Camio_Escombraries " & Exception_Name(E) & ", causada per: " & Exception_Message(E));
                           Final_Simulador.Indicar_Fi_Simulacio;
      end Tasca_Camio_Escombraries;

      -----------------------------
      -- Tasca_Manegador_Cruilla --
      -----------------------------

      task body Tasca_Manegador_Cruilla is
         Minuts_Periode : Positive;
         Duracio_Periode : Time_Span;
         Temps_Actual : Time;
         Seguent_Periode : Time;
         Color_H : Estat_Semafor;
         Color_V : Estat_Semafor;
         Fi : Boolean := False;
         D : Direccio;
         Color : Estat_Semafor;
         Uts : Positive;
      begin
         Minuts_Periode := Obtenir_Periode_Semafors(C);
         Uts := Unitat_Temps_Simulacio(C);
         Duracio_Periode := MilliSeconds(((Minuts_Periode * 60) / Uts) * 1000);
         Color_H := Estat_Inici_H;
         if Color_H = Verd then
            Color_V := Vermell;
         else
            Color_V := Verd;
         end if;
         Inicialitzar_Cruilla(C,Fila,Columna,Color_H,Color_V);
         Principal.Tasca_Preparada;
         Seguent_Periode := Clock;
         while not Fi loop
            select
               Final_Simulador.Comprovar_Final;
               Fi := True;
            then abort
               while not Fi loop
                  select
                     Sincro_Cruilles(Fila,Columna).Esperar_Modificacio_Periode(D,Color);
		               Alternar_Semafors(C,Fila,Columna);
                     Canviar_Semafors_Interficie(C,Fila,Columna,D,Color);
                     Temps_Actual := Clock;
		               Seguent_Periode := Temps_Actual + Duracio_Periode;
                  or
                     delay until Seguent_Periode;
		               Alternar_Semafors(C,Fila,Columna);
                     Color := Estat_Actual_Semafor (C,Fila,Columna,Horitzontal);
                     Canviar_Semafors_Interficie(C,Fila,Columna,Horitzontal,Color);
		               Temps_Actual := Clock;
		               Seguent_Periode := Temps_Actual + Duracio_Periode;
                  end select;
               end loop;
            end select;
         end loop;
      exception
         when E: others => Put_Line("Excepcio en Tasca_Manegador_Cruilla " & Exception_Name(E) & ", causada per: " & Exception_Message(E));
                           Final_Simulador.Indicar_Fi_Simulacio;
      end Tasca_Manegador_Cruilla;
      
      ------------------------------
      -- Tasca_Controlador_Ciutat --
      ------------------------------ 

      task body Tasca_Controlador_Ciutat is

         -- Tasca_Control_Modificacio_Periode:
         -- Tasca encarregada de modificar el periode del semafor de la direccio i cruilla indicades,
	   -- en el moment en que no hi hagi ningu creuant en l'altra direccio per la mateixa cruilla, i 
	   -- sempre i quant mentre s'espera no s'hagin alternat els semafors de l'actual cruilla.
	   task type Tasca_Control_Modificacio_Periode (I: Positive; J: Positive; Dir: Direccio);
	   task body Tasca_Control_Modificacio_Periode is
	   begin
	      while Vehicles_Creuant.Comprovar_Vehicles_Creuant(I,J) = true loop
	         null;
	      end loop;
            if Estat_Actual_Semafor (C,I,J,Dir) = Vermell then
	         Sincro_Cruilles(I,J).Nova_Modificacio_Periode(Dir,Verd);
	      end if;
	   end Tasca_Control_Modificacio_Periode;
         type Acces_Tasca_Modificacio_Periode is access Tasca_Control_Modificacio_Periode;
	 
         Fi : Boolean := False;
         OK : Boolean;
         Dades_Avaria : Info_Avaria;
         Id_Vehicle : Positive;
         Tipus : Tipus_Vehicle;
         Uts : Positive;
         Control_Trafic_Activat : Boolean;
         Seguent_Periode : Time;
         Duracio_Periode : Time_Span;
	   Temps_Actual : Time;
         Cua_Semafor : Natural;
	   Divisor_Minuts_Controlador : Positive;
	   Minuts_Semafors : Positive;
	   Modificador_Cruilla : Acces_Tasca_Modificacio_Periode;
	 
      begin
         Control_Trafic_Activat := Control_Trafic_Ciutat(C);
	   Divisor_Minuts_Controlador := Obtenir_Temps_Control_Trafic(C);
	   Minuts_Semafors := Obtenir_Periode_semafors(C);
         Uts := Unitat_Temps_Simulacio(C);
         -- Control de trafic cada (Periode_Semafors / Divisor_Controlador) minuts
	   Duracio_Periode := MilliSeconds((((Minuts_Semafors * 60) / Divisor_Minuts_Controlador) / Uts) * 1000);  
	   Seguent_Periode := Clock;
         accept Iniciar_Tasca;
         while not Fi loop
            select
               Final_Simulador.Comprovar_Final;
               Fi := True;
            then abort
               -- Realitzar control de trafic mentre no es reben notificacions 
               -- de vehicles espatllats. 
               while not Fi loop
                  select
                     Espatllats.Gestio_Vehicles_Espatllats;
                  then abort
                     if Control_Trafic_Activat then
                        -- Es realitza control de trafic durant la execucio de la simulacio
                        delay until Seguent_Periode;
                        for I in 1..Num_Carrers(C,Horitzontal) loop
                           for J in 1..Num_Carrers(C,Vertical) loop
                              Cua_Semafor := Vehicles_Aturats_Semafor (C,I,J,Horitzontal);
                              if Cua_Semafor > 0 then
                                 Modificador_Cruilla := new Tasca_Control_Modificacio_Periode(I,J,Horitzontal);
                              else
                                 Cua_Semafor := Vehicles_Aturats_Semafor (C,I,J,Vertical);
                                 if Cua_Semafor > 0 then
                                    Modificador_Cruilla := new Tasca_Control_Modificacio_Periode(I,J,Vertical);
                                 end if;
                              end if;
                           end loop;
                        end loop;
			      Temps_Actual := Clock;
			      Seguent_Periode := Temps_Actual + Duracio_Periode;
                     else
  		     	      delay until Seguent_Periode;
                     end if;
                  end select;
               end loop;
            end select;
         end loop;   
      exception
         when E: others => Put_Line("Excepcio en Tasca_Controlador_Ciutat " & Exception_Name(E) & ", causada per: " & Exception_Message(E));
                           Final_Simulador.Indicar_Fi_Simulacio;
      end Tasca_Controlador_Ciutat;

      ------------------------------------
      -- Tasca_Actualitzador_Interficie --
      ------------------------------------

      task body Actualitzador_Interficie is
         Fi : Boolean := False;
         Dades : Info_Accio;
      begin
         accept Iniciar_Tasca;
         while not Fi loop
            select
               Final_Simulador.Comprovar_Final;
               Fi := True;
            then abort
               loop
                  Esperar_Accions_Interficie(C,Dades);
                  Actualitzar_Interficie(Dades);
               end loop;
            end select;
         end loop;
         while Accions_Per_Tractar(C) loop
            Esperar_Accions_Interficie(C,Dades);
            Actualitzar_Interficie(Dades);
         end loop;
         Principal.Tasca_Finalitzada;
      Exception
         when E: others => Put_Line("Excepcio en Actualitzador_Interficie " & Exception_Name(E) & ", causada per: " & Exception_Message(E));
                           Final_Simulador.Indicar_Fi_Simulacio;
      end Actualitzador_Interficie;


      Espatllats : Vehicles_Espatllats;

      ---------------------
      -- Tasca_Principal --
      ---------------------
      
      task body Tasca_Principal is
         Fila, Columna : Positive;
         Nombre_Vehicles : Natural;
         Tasques_Finalitzades : Natural;
         Ok : Boolean;
         Llavor : Generator;
         Controlador : Acces_Tasca_Controlador;
         Escombraries : Acces_Tasca_Escombraries;
         Semafors : Conjunt_Semafors;
         Grua : Acces_Tasca_Grua;
         Cotxes : Conjunt_Cotxes;
         Ciclomotors : Conjunt_Ciclomotors;
         Camions : Conjunt_Camions;
         Bicicletes : Conjunt_Bicicletes;
         Comunicador_Grafic : Acces_Actualitzador_Interficie;
         Final_Simulador : Finalitzacio_Simulacio;
         Vehicles_Creuant : Vehicles_Creuant_Cruilla;
      begin

         -- Inicialitzacions de objectes protegits

         for I in 1..Num_Carrers(C,Horitzontal) loop
            for J in 1..Num_Carrers(C,Vertical) loop
               Sincro_Cruilles(I,J) := new Sincronitzacions_Semafors_Protegides;
            end loop;
         end loop;

         Final_Simulador := new Finalitzacio_Simulacio_Protegida;
         Vehicles_Creuant := new Vehicles_Creuant_Cruilla_Protegits(Num_Carrers(C,Horitzontal),Num_Carrers(C,Vertical));
         Vehicles_Creuant.Inicialitzar_Creuaments;
	 
         -- Creacio de les tasques
         for I in 1..Cotxes'Length loop
            Cotxes(I) := new Tasca_Vehicle(I,Cotxe,Principal,Espatllats,Vehicles_Creuant);
         end loop;
         for I in 1..Ciclomotors'Length loop
            Ciclomotors(I) := new Tasca_Vehicle(I,Ciclomotor,Principal,Espatllats,Vehicles_Creuant);
         end loop;
         for I in 1..Camions'Length loop
            Camions(I) := new Tasca_Vehicle(I,Camio,Principal,Espatllats,Vehicles_Creuant);
         end loop;
         for I in 1..Bicicletes'Length loop
            Bicicletes(I) := new Tasca_Vehicle(I,Bicicleta,Principal,Espatllats,Vehicles_Creuant);
         end loop;

         
         for I in 1..Semafors'Length loop
            Obtenir_Coordenades_Cruilla(C,I,Fila,Columna);
            Semafors(I) := new Tasca_Manegador_Cruilla(Fila,Columna,Verd,Principal,
                                                       Final_Simulador);
         end loop;
         
   
         Nombre_Vehicles := Cotxes'Length + Ciclomotors'Length +
                            Camions'Length + Bicicletes'Length;
         Grua := new Tasca_Grua(Principal,Espatllats,Final_Simulador,Nombre_Vehicles,Vehicles_Creuant);
         Escombraries := new Tasca_Camio_Escombraries(Final_Simulador,Vehicles_Creuant);
         Controlador := new Tasca_Controlador_Ciutat(Principal,Espatllats,Final_Simulador,Vehicles_Creuant);
         Comunicador_Grafic := new Actualitzador_Interficie(Final_Simulador);
         Comunicador_Grafic.Iniciar_Tasca;

         -- Esperar inicialitzacio ciutat
         for I in 1..Conjunt_Semafors'Length loop
            accept Tasca_Preparada;
         end loop;


         --Sincronitzar inici tasques
         for I in 1..Cotxes'Length loop
            Cotxes(I).Iniciar_Tasca;
         end loop;
         for I in 1..Ciclomotors'Length loop
            Ciclomotors(I).Iniciar_Tasca;
         end loop;
         for I in 1..Camions'Length loop
            Camions(I).Iniciar_Tasca;
         end loop;
         for I in 1..Bicicletes'Length loop
            Bicicletes(I).Iniciar_Tasca;
         end loop;        
         Grua.Iniciar_Tasca;
         Escombraries.Iniciar_Tasca;
         Controlador.Iniciar_Tasca;


         -- Esperar fi tasques vehicle
         Tasques_Finalitzades := 0;
         while Tasques_Finalitzades < Nombre_Vehicles loop
            accept Tasca_Finalitzada;
            Tasques_Finalitzades := Tasques_Finalitzades + 1;
         end loop;

         Final_Simulador.Anticipar_Fi_Simulacio;
         accept Tasca_Finalitzada;          -- Tasca_Camio_Escombraries finalitzada
         accept Tasca_Finalitzada;          -- Tasca_Grua finalitzada

         -- Indicar fi simulacio
         Final_Simulador.Indicar_Fi_Simulacio;


         accept Tasca_Finalitzada;          -- Actualitzador d'interficie finalitzat

      exception
         when E: others => Put_Line("Excepcio en Tasca_Principal " & Exception_Name(E));
                           Final_Simulador.Indicar_Fi_Simulacio;
      end Tasca_Principal;

   begin

      Reset(Llavor);
      Espatllats := new Vehicles_Espatllats_Protegits(Nombre_Vehicles(C,Cotxe),Nombre_Vehicles(C,Ciclomotor),
                                                      Nombre_Vehicles(C,Camio),Nombre_Vehicles(C,Bicicleta));
      Principal := new Tasca_Principal;
      
   end Execucio_Tasques;


   -------------------------
   -- Vehicles_Espatllats --
   -------------------------

   protected body Vehicles_Espatllats_Protegits is
      entry Nou_Vehicle_Espatllat (Id: in Positive; Tipus: in Tipus_Vehicle;
                                       Pos_X_Actual, Pos_Y_Actual: in Positive;
                                       Carrer_Actual: Positive; Direccio_Actual: Direccio) when Algun_Vehicle = 0 is
      begin
         Id_Vehicle := Id;
         Clase_Vehicle := Tipus;
         Ubicacio_X := Pos_X_Actual;
         Ubicacio_Y := Pos_Y_Actual;
         Carrer := Carrer_Actual;
         Orientacio := Direccio_Actual;
         Algun_Vehicle := Algun_Vehicle + 1;
      end Nou_Vehicle_Espatllat;
      
      entry Gestio_Vehicles_Espatllats when Algun_Vehicle > 0 is
         Dades_Avaria : Info_Avaria;
      begin
         Dades_Avaria.Id_Vehicle := Id_Vehicle;
         Dades_Avaria.Tipus := Clase_Vehicle;
         Dades_Avaria.Pos_X := Ubicacio_X;
         Dades_Avaria.Pos_Y := Ubicacio_Y;
         Dades_Avaria.Num_Carrer := Carrer;
         Dades_Avaria.Orientacio := Orientacio;
         Afegir_Avaria (Avaries_Vehicles,Dades_Avaria,OK);
         --Vehicles_Espatllats := Vehicles_Espatllats + 1;
         Grua_Requerida := Grua_Requerida + 1;
         Algun_Vehicle := Algun_Vehicle - 1;
      end Gestio_Vehicles_Espatllats;
      
      entry Activar_Grua (Dades_Avaria: out Info_Avaria) when Grua_Requerida > 0 is
      begin
         Obtenir_Avaria (Avaries_Vehicles,Dades_Avaria,OK);
         Grua_Requerida := Grua_Requerida - 1;
      end Activar_Grua;

   end Vehicles_Espatllats_Protegits;

   -------------------------------
   -- Sincronitzacions_Semafors --
   -------------------------------

   protected body Sincronitzacions_Semafors_Protegides is

      procedure Nova_Modificacio_Periode (D: in Direccio; Color_Semafor: in Estat_Semafor) is
      begin   
           Modificacio.Dir := D;
           Modificacio.Color := Color_Semafor;
           Alguna_Modificacio := Alguna_Modificacio + 1;          
      end Nova_Modificacio_Periode;

      entry Esperar_Modificacio_Periode ( 
                                         D: out Direccio; Color_Semafor: out Estat_Semafor) when Alguna_Modificacio > 0 is
      begin
            D := Modificacio.Dir;
            Color_Semafor := Modificacio.Color;
            Alguna_Modificacio := Alguna_Modificacio - 1;
      end Esperar_Modificacio_Periode;
   end Sincronitzacions_Semafors_Protegides;

   ----------------------------
   -- Finalitzacio_Simulacio --
   ----------------------------

   protected body Finalitzacio_Simulacio_Protegida is
      procedure Indicar_Fi_Simulacio is
      begin
         Fi := True;
      end Indicar_Fi_Simulacio;
      
      entry Comprovar_Final when Fi is
      begin
         null;
      end Comprovar_Final;
      
      function Fi_Anticipada return Boolean is
      begin
         return Fi_Propera;
      end Fi_Anticipada;

      entry Comprovar_Fi_Anticipada when Fi_Propera is
      begin
         null;
      end Comprovar_Fi_Anticipada;      

      procedure Anticipar_Fi_Simulacio is
      begin
         Fi_Propera := True;
      end Anticipar_Fi_Simulacio;
   end Finalitzacio_Simulacio_Protegida;

   protected body Vehicles_Creuant_Cruilla_Protegits is
      procedure Inicialitzar_Creuaments is
      begin
         for I in 1..Num_Files loop
            for J in 1..Num_Columnes loop
               Vehicles_Creuant(I,J) := 0;
            end loop;
         end loop;
      end Inicialitzar_Creuaments;
   
      procedure Nou_Vehicle_Creuant(I: in Positive; J: in Positive) is
      begin
         Vehicles_Creuant(I,J) := Vehicles_Creuant(I,J) + 1;
      end Nou_Vehicle_Creuant;
      
      procedure Vehicle_Surt_De_Cruilla(I: in Positive; J: in Positive) is
      begin
         Vehicles_Creuant(I,J) := Vehicles_Creuant(I,J) - 1;
      end Vehicle_Surt_De_Cruilla;
      
      function Comprovar_Vehicles_Creuant(I: in Positive; J: in Positive) return Boolean is
      begin
         if Vehicles_Creuant(I,J) > 0 then
            return True;
         else
            return False;
         end if;
      end Comprovar_Vehicles_Creuant;
   end Vehicles_Creuant_Cruilla_Protegits;
   
end Tasques;

