
------------------------------------------------------------------------------------------
-- Practica de Enginyeria del Software II / Curs 2003 - 2004                            --
-- Simulador i Controlador de Transit en Temps Real                                     --
--                                                                                      --
-- Tasques.ads: Fitxer descriptor del modul Tasques, amb les especificacions dels       --
--              procediments, tipus de dades i objectes protegits que s'empren en la    --
--              implementacio d'aquest package.                                         --
-- Realitzat per: Francisco Jose Aguilar Celdran                                        --
--                Sergio Blanco Cuaresma                                                --
------------------------------------------------------------------------------------------


with Tipus_Comuns; use Tipus_Comuns;
with Ciutats; use Ciutats;
with Cruilles; use Cruilles;
with Avaries; use Avaries;

package Tasques is
   
   procedure Execucio_Tasques (C: in out Ciutat);

private

   type Fi_Cotxes is array (Integer range <>) of Boolean;
   type Fi_Ciclomotors is array (Integer range <>) of Boolean;
   type Fi_Camions is array (Integer range <>) of Boolean;
   type Fi_Bicicletes is array (Integer range <>) of Boolean;

   protected type Vehicles_Espatllats_Protegits (Num_Cotxes, Num_Ciclos, Num_Camions,
                                                 Num_Bicis: Natural) is
      entry Nou_Vehicle_Espatllat (Id: in Positive; Tipus: in Tipus_Vehicle;
                                   Pos_X_Actual, Pos_Y_Actual: in Positive;
                                   Carrer_Actual: Positive; Direccio_Actual: Direccio);
      entry Gestio_Vehicles_Espatllats;
      entry Activar_Grua (Dades_Avaria: out Info_Avaria);
   private
      Avaries_Vehicles : Avaria;
      Id_Vehicle : Positive;
      Clase_Vehicle : Tipus_Vehicle;
      Ubicacio_X : Positive;
      Ubicacio_Y : Positive;
      Carrer : Positive;
      Orientacio : Direccio;
      Algun_Vehicle : Natural := 0;
      Grua_Requerida : Natural := 0;
      OK : Boolean := False;
   end Vehicles_Espatllats_Protegits;
   type Vehicles_Espatllats is access Vehicles_Espatllats_Protegits;

   type Info_Periode is record
      Estat : Boolean;
      Dir : Direccio;
      Color : Estat_Semafor;
   end record;

   protected type Sincronitzacions_Semafors_Protegides is
      procedure Nova_Modificacio_Periode (D: in Direccio; Color_Semafor: in Estat_Semafor);
      entry Esperar_Modificacio_Periode (D: out Direccio; Color_Semafor: out Estat_Semafor);
   private
      Modificacio : Info_Periode; 
      Alguna_Modificacio : Integer := 0;
   end Sincronitzacions_Semafors_Protegides;
   type Sincronitzacions_Semafors is access Sincronitzacions_Semafors_Protegides; 

   protected type Finalitzacio_Simulacio_Protegida is
      procedure Indicar_Fi_Simulacio;
      entry Comprovar_Final;
      function Fi_Anticipada return Boolean;
      entry Comprovar_Fi_Anticipada;
      procedure Anticipar_Fi_Simulacio;
   private
      Fi : Boolean := False;
      Fi_Propera : Boolean := False;
   end Finalitzacio_Simulacio_Protegida;
   type Finalitzacio_Simulacio is access Finalitzacio_Simulacio_Protegida;

   type Taula_Vehicles_Creuant is array (Positive range <>,Positive range <>) of Natural;
   protected type Vehicles_Creuant_Cruilla_Protegits(Files: Positive; Columnes: Positive) is
      procedure Inicialitzar_Creuaments;
      procedure Nou_Vehicle_Creuant(I: in Positive; J: in Positive);
      procedure Vehicle_Surt_De_Cruilla(I: in Positive; J: in Positive);
      function Comprovar_Vehicles_Creuant(I: in Positive; J: in Positive) return Boolean;
   private
      Vehicles_Creuant : Taula_Vehicles_Creuant(1..Files,1..Columnes);
      Num_Files : Positive := Files;
      Num_Columnes : Positive := Columnes;
   end Vehicles_Creuant_Cruilla_Protegits;
   type Vehicles_Creuant_Cruilla is access Vehicles_Creuant_Cruilla_Protegits;
   
end Tasques;

